@@ -1,15 +1,16 @@
-#include "bpforc.h"
 #include "codegen_llvm.h"
-#include "ast.h"
-#include "parser.tab.hh"
 #include "arch/arch.h"
+#include "ast.h"
+#include "bpforc.h"
+#include "parser.tab.hh"
+#include "tracepoint_format_parser.h"
 #include "types.h"
 #include "utils.h"
-#include <time.h>
+#include <algorithm>
 #include <arpa/inet.h>
-#include "tracepoint_format_parser.h"
+#include <csignal>
+#include <ctime>
 
-#include <llvm/Support/raw_os_ostream.h>
 #include <llvm/Support/TargetRegistry.h>
 #include <llvm/IR/Constants.h>
 #include <llvm/IR/LegacyPassManager.h>
@@ -27,13 +28,17 @@ void CodegenLLVM::visit(Integer &integer)
 
 void CodegenLLVM::visit(PositionalParameter &param)
 {
-  switch (param.ptype) {
+  switch (param.ptype)
+  {
     case PositionalParameterType::positional:
       {
         std::string pstr = bpftrace_.get_param(param.n, param.is_in_str);
-        if (bpftrace_.is_numeric(pstr)) {
+        if (is_numeric(pstr))
+        {
           expr_ = b_.getInt64(std::stoll(pstr));
-        } else {
+        }
+        else
+        {
           Constant *const_str = ConstantDataArray::getString(module_->getContext(), pstr, true);
           AllocaInst *buf = b_.CreateAllocaBPF(ArrayType::get(b_.getInt8Ty(), pstr.length() + 1), "str");
           b_.CreateMemSet(buf, b_.getInt8(0), pstr.length() + 1, 1);
@@ -82,10 +87,15 @@ void CodegenLLVM::visit(Builtin &builtin)
   }
   else if (builtin.ident == "elapsed")
   {
-    struct timespec ts;
-    // this time will be calculated during codegen and baked into the BPF
-    clock_gettime(CLOCK_MONOTONIC, &ts);
-    expr_ = b_.CreateSub(b_.CreateGetNs(), b_.getInt64(1000000000ULL * ts.tv_sec + ts.tv_nsec));
+    AllocaInst *key = b_.CreateAllocaBPF(b_.getInt64Ty(), "elapsed_key");
+    b_.CreateStore(b_.getInt64(0), key);
+
+    auto &map = bpftrace_.elapsed_map_;
+    auto type = SizedType(Type::integer, 8);
+    auto start = b_.CreateMapLookupElem(map->mapfd_, key, type);
+    expr_ = b_.CreateSub(b_.CreateGetNs(), start);
+    // start won't be on stack, no need to LifeTimeEnd it
+    b_.CreateLifetimeEnd(key);
   }
   else if (builtin.ident == "kstack" || builtin.ident == "ustack")
   {
@@ -175,11 +185,23 @@ void CodegenLLVM::visit(Builtin &builtin)
         b_.getInt64Ty(),
         b_.CreateGEP(ctx_, b_.getInt64(offset * sizeof(uintptr_t))),
         builtin.ident);
+
+    if (builtin.type.type == Type::usym)
+    {
+      AllocaInst *buf = b_.CreateAllocaBPF(builtin.type, "func");
+      b_.CreateMemSet(buf, b_.getInt8(0), builtin.type.size, 1);
+      Value *pid = b_.CreateLShr(b_.CreateGetPidTgid(), 32);
+      Value *addr_offset = b_.CreateGEP(buf, b_.getInt64(0));
+      Value *pid_offset = b_.CreateGEP(buf, {b_.getInt64(0), b_.getInt64(8)});
+      b_.CreateStore(expr_, addr_offset);
+      b_.CreateStore(pid, pid_offset);
+      expr_ = buf;
+    }
   }
   else if (!builtin.ident.compare(0, 4, "sarg") && builtin.ident.size() == 5 &&
       builtin.ident.at(4) >= '0' && builtin.ident.at(4) <= '9')
   {
-    int sp_offset = arch::offset("sp");
+    int sp_offset = arch::sp_offset();
     if (sp_offset == -1)
     {
       std::cerr << "negative offset for stack pointer" << std::endl;
@@ -214,6 +236,15 @@ void CodegenLLVM::visit(Builtin &builtin)
   {
     expr_ = ctx_;
   }
+  else if (builtin.ident == "cpid")
+  {
+    pid_t cpid = bpftrace_.child_pid();
+    if (cpid < 1) {
+      std::cerr << "BUG: Invalid cpid: " << cpid << std::endl;
+      abort();
+    }
+    expr_ = b_.getInt32(cpid);
+  }
   else if (builtin.ident == "ctx")
   {
     // undocumented builtin: for debugging
@@ -268,7 +299,7 @@ void CodegenLLVM::visit(Call &call)
     AllocaInst *newval = b_.CreateAllocaBPF(map.type, map.ident + "_val");
 
     // Store the max of (0xffffffff - val), so that our SGE comparison with uninitialized
-    // elements will always store on the first occurrance. Revent this later when printing.
+    // elements will always store on the first occurrence. Revent this later when printing.
     Function *parent = b_.GetInsertBlock()->getParent();
     call.vargs->front()->accept(*this);
     // promote int to 64-bit
@@ -448,10 +479,14 @@ void CodegenLLVM::visit(Call &call)
   }
   else if (call.func == "uaddr")
   {
-   uint64_t addr;
     auto &name = static_cast<String&>(*call.vargs->at(0)).str;
-    addr = bpftrace_.resolve_uname(name, current_attach_point_->target);
-    expr_ = b_.getInt64(addr);
+    struct symbol sym = {};
+    int err =
+        bpftrace_.resolve_uname(name, &sym, current_attach_point_->target);
+    if (err < 0 || sym.address == 0)
+      throw std::runtime_error("Could not resolve symbol: " +
+                               current_attach_point_->target + ":" + name);
+    expr_ = b_.getInt64(sym.address);
   }
   else if (call.func == "cgroupid")
   {
@@ -474,7 +509,8 @@ void CodegenLLVM::visit(Call &call)
     // arg0
     b_.SetInsertPoint(notzero);
     b_.CreateStore(b_.getInt64(asyncactionint(AsyncAction::join)), perfdata);
-    b_.CreateStore(b_.getInt64(join_id_), b_.CreateGEP(perfdata, {b_.getInt64(8)}));
+    b_.CreateStore(b_.getInt64(join_id_),
+                   b_.CreateGEP(perfdata, b_.getInt64(8)));
     join_id_++;
     AllocaInst *arr = b_.CreateAllocaBPF(b_.getInt64Ty(), call.func+"_r0");
     b_.CreateProbeRead(arr, 8, expr_);
@@ -536,13 +572,19 @@ void CodegenLLVM::visit(Call &call)
     Value *af_type;
 
     auto inet = call.vargs->at(0);
-    if (call.vargs->size() == 1) {
-      if (inet->type.type == Type::integer || inet->type.size == 4) {
+    if (call.vargs->size() == 1)
+    {
+      if (inet->type.type == Type::integer || inet->type.size == 4)
+      {
         af_type = b_.getInt64(AF_INET);
-      } else {
+      }
+      else
+      {
         af_type = b_.getInt64(AF_INET6);
       }
-    } else {
+    }
+    else
+    {
       inet = call.vargs->at(1);
       call.vargs->at(0)->accept(*this);
       af_type = b_.CreateIntCast(expr_, b_.getInt64Ty(), true);
@@ -553,9 +595,12 @@ void CodegenLLVM::visit(Call &call)
     b_.CreateMemSet(inet_offset, b_.getInt8(0), 16, 1);
 
     inet->accept(*this);
-    if (inet->type.type == Type::array) {
-      b_.CreateProbeRead(reinterpret_cast<AllocaInst *>(inet_offset), inet->type.size, expr_);
-    } else {
+    if (inet->type.type == Type::array)
+    {
+      b_.CreateProbeRead(static_cast<AllocaInst *>(inet_offset), inet->type.size, expr_);
+    }
+    else
+    {
       b_.CreateStore(b_.CreateIntCast(expr_, b_.getInt32Ty(), false), inet_offset);
     }
 
@@ -702,7 +747,71 @@ void CodegenLLVM::visit(Call &call)
     }
     expr_ = stackid;
   }
+  else if (call.func == "signal") {
+    // int bpf_send_signal(u32 sig)
+    auto &arg = *call.vargs->at(0);
+    if (arg.type.type == Type::string) {
+      auto signame = static_cast<String&>(arg).str;
+      int sigid = signal_name_to_num(signame);
+      // Should be caught in semantic analyser
+      if (sigid < 1) {
+        std::cerr << "BUG: Invalid signal ID for \"" << signame << "\"";
+        abort();
+      }
+      b_.CreateSignal(b_.getInt32(sigid));
+      return;
+    }
+    arg.accept(*this);
+    if (arg.is_literal) {
+      b_.CreateSignal(b_.getInt32(static_cast<Integer&>(arg).n));
+    }
+    else {
+      expr_ = b_.CreateIntCast(expr_, b_.getInt32Ty(), arg.type.is_signed);
+      b_.CreateSignal(expr_);
+    }
+  }
+  else if (call.func == "strncmp") {
+    uint64_t size = static_cast<Integer *>(call.vargs->at(2))->n;
+    const auto& left_arg = call.vargs->at(0);
+    const auto& right_arg = call.vargs->at(1);
 
+    // If one of the strings is fixed, we can avoid storing the
+    // literal in memory by calling a different function.
+    if (right_arg->is_literal) {
+      left_arg->accept(*this);
+      Value *left_string = expr_;
+      const auto& string_literal = static_cast<String *>(right_arg)->str;
+      expr_ = b_.CreateStrncmp(left_string, string_literal, size, false);
+      if (!left_arg->is_variable && dyn_cast<AllocaInst>(left_string))
+        b_.CreateLifetimeEnd(left_string);
+    } else if (left_arg->is_literal) {
+      right_arg->accept(*this);
+      Value *right_string = expr_;
+      const auto& string_literal = static_cast<String *>(left_arg)->str;
+      expr_ = b_.CreateStrncmp(right_string, string_literal, size, false);
+      if (!right_arg->is_variable && dyn_cast<AllocaInst>(right_string))
+        b_.CreateLifetimeEnd(right_string);
+    } else {
+      right_arg->accept(*this);
+      Value *right_string = expr_;
+      left_arg->accept(*this);
+      Value *left_string = expr_;
+      expr_ = b_.CreateStrncmp(left_string, right_string, size, false);
+      if (!left_arg->is_variable && dyn_cast<AllocaInst>(left_string))
+        b_.CreateLifetimeEnd(left_string);
+      if (!right_arg->is_variable && dyn_cast<AllocaInst>(right_string))
+        b_.CreateLifetimeEnd(right_string);
+    }
+  }
+  else if (call.func == "override")
+  {
+    // int bpf_override(struct pt_regs *regs, u64 rc)
+    // returns: 0
+    auto &arg = *call.vargs->at(0);
+    arg.accept(*this);
+    expr_ = b_.CreateIntCast(expr_, b_.getInt64Ty(), arg.type.is_signed);
+    b_.CreateOverrideReturn(ctx_, expr_);
+  }
   else
   {
     std::cerr << "missing codegen for function \"" << call.func << "\"" << std::endl;
@@ -754,8 +863,11 @@ void CodegenLLVM::visit(Binop &binop)
 
     std::string string_literal("");
 
-    bool inverse = binop.op == bpftrace::Parser::token::NE;
+    // strcmp returns 0 when strings are equal
+    bool inverse = binop.op == bpftrace::Parser::token::EQ;
 
+    // If one of the strings is fixed, we can avoid storing the
+    // literal in memory by calling a different function.
     if (binop.right->is_literal)
     {
       binop.left->accept(*this);
@@ -776,7 +888,8 @@ void CodegenLLVM::visit(Binop &binop)
       binop.left->accept(*this);
       Value * left_string = expr_;
 
-      expr_ = b_.CreateStrcmp(left_string, right_string, inverse);
+      size_t len = std::min(binop.left->type.size, binop.right->type.size);
+      expr_ = b_.CreateStrncmp(left_string, right_string, len + 1, inverse);
     }
   }
   else
@@ -820,7 +933,11 @@ void CodegenLLVM::visit(Binop &binop)
       case bpftrace::Parser::token::MUL:   expr_ = b_.CreateMul    (lhs, rhs); break;
       case bpftrace::Parser::token::DIV:   expr_ = b_.CreateUDiv   (lhs, rhs); break;
       case bpftrace::Parser::token::MOD: {
-        expr_ = do_signed ? b_.CreateSRem(lhs, rhs) : b_.CreateURem(lhs, rhs);
+        // Always do an unsigned modulo operation here even if `do_signed`
+        // is true. bpf instruction set does not support signed division.
+        // We already warn in the semantic analyser that signed modulo can
+        // lead to undefined behavior (because we will treat it as unsigned).
+        expr_ = b_.CreateURem(lhs, rhs);
         break;
       }
       case bpftrace::Parser::token::BAND:  expr_ = b_.CreateAnd    (lhs, rhs); break;
@@ -844,8 +961,8 @@ static bool unop_skip_accept(Unop &unop)
 {
   if (unop.expr->type.type == Type::integer)
   {
-    if (unop.op == bpftrace::Parser::token::PLUSPLUS ||
-        unop.op == bpftrace::Parser::token::MINUSMINUS)
+    if (unop.op == bpftrace::Parser::token::INCREMENT ||
+        unop.op == bpftrace::Parser::token::DECREMENT)
       return unop.expr->is_map || unop.expr->is_variable;
   }
 
@@ -867,10 +984,10 @@ void CodegenLLVM::visit(Unop &unop)
       } break;
       case bpftrace::Parser::token::BNOT: expr_ = b_.CreateNot(expr_); break;
       case bpftrace::Parser::token::MINUS: expr_ = b_.CreateNeg(expr_); break;
-      case bpftrace::Parser::token::PLUSPLUS:
-      case bpftrace::Parser::token::MINUSMINUS:
+      case bpftrace::Parser::token::INCREMENT:
+      case bpftrace::Parser::token::DECREMENT:
       {
-        bool is_increment = unop.op == bpftrace::Parser::token::PLUSPLUS;
+        bool is_increment = unop.op == bpftrace::Parser::token::INCREMENT;
 
         if (unop.expr->is_map)
         {
@@ -994,11 +1111,15 @@ void CodegenLLVM::visit(Ternary &ternary)
     b_.SetInsertPoint(left_block);
     ternary.left->accept(*this);
     b_.CREATE_MEMCPY(buf, expr_, ternary.type.size, 1);
+    if (!ternary.left->is_variable && dyn_cast<AllocaInst>(expr_))
+      b_.CreateLifetimeEnd(expr_);
     b_.CreateBr(done);
 
     b_.SetInsertPoint(right_block);
     ternary.right->accept(*this);
     b_.CREATE_MEMCPY(buf, expr_, ternary.type.size, 1);
+    if (!ternary.right->is_variable && dyn_cast<AllocaInst>(expr_))
+      b_.CreateLifetimeEnd(expr_);
     b_.CreateBr(done);
 
     b_.SetInsertPoint(done);
@@ -1079,6 +1200,18 @@ void CodegenLLVM::visit(FieldAccess &acc)
       b_.CreateProbeRead(dst, field.type.size, src);
       expr_ = dst;
     }
+    else if (field.type.type == Type::integer && field.is_bitfield)
+    {
+      AllocaInst *dst = b_.CreateAllocaBPF(field.type, type.cast_type + "." + acc.field);
+      // memset so verifier doesn't complain about reading uninitialized stack
+      b_.CreateMemSet(dst, b_.getInt8(0), field.type.size, 1);
+      b_.CreateProbeRead(dst, field.bitfield.read_bytes, src);
+      Value *raw = b_.CreateLoad(dst);
+      Value *shifted = b_.CreateLShr(raw, field.bitfield.access_rshift);
+      Value *masked = b_.CreateAnd(shifted, field.bitfield.mask);
+      expr_ = masked;
+      b_.CreateLifetimeEnd(dst);
+    }
     else
     {
       AllocaInst *dst = b_.CreateAllocaBPF(field.type, type.cast_type + "." + acc.field);
@@ -1196,6 +1329,8 @@ void CodegenLLVM::visit(AssignVarStatement &assignment)
   else
   {
     b_.CREATE_MEMCPY(variables_[var.ident], expr_, var.type.size, 1);
+    if (!assignment.expr->is_variable && dyn_cast<AllocaInst>(expr_))
+      b_.CreateLifetimeEnd(expr_);
   }
 }
 
@@ -1285,17 +1420,15 @@ void CodegenLLVM::visit(Probe &probe)
       {b_.getInt8PtrTy()}, // struct pt_regs *ctx
       false);
 
-  // needed for uaddr() calls and usdt probes:
-  for (auto &attach_point : *probe.attach_points) {
+  // Probe has at least one attach point (required by the parser)
+  auto &attach_point = (*probe.attach_points)[0];
 
-    // All usdt probes need expansion to be able to read arguments
-    if(probetype(attach_point->provider) == ProbeType::usdt)
-      probe.need_expansion = true;
+  // All usdt probes need expansion to be able to read arguments
+  if (probetype(attach_point->provider) == ProbeType::usdt)
+    probe.need_expansion = true;
+
+  current_attach_point_ = attach_point;
 
-    current_attach_point_ = attach_point;
-    // TODO: semantic analyser should ensure targets are equal when uaddr() is used
-    break;
-  }
   /*
    * Most of the time, we can take a probe like kprobe:do_f* and build a
    * single BPF program for that, called "s_kprobe:do_f*", and attach it to
@@ -1329,9 +1462,11 @@ void CodegenLLVM::visit(Probe &probe)
      * We begin by saving state that gets changed by the codegen pass, so we
      * can restore it for the next pass (printf_id_, time_id_).
      */
-    int starting_printf_id_ = printf_id_;
-    int starting_time_id_ = time_id_;
-    int starting_join_id_ = join_id_;
+    int starting_printf_id = printf_id_;
+    int starting_cat_id = cat_id_;
+    int starting_system_id = system_id_;
+    int starting_time_id = time_id_;
+    int starting_join_id = join_id_;
 
     for (auto attach_point : *probe.attach_points) {
       current_attach_point_ = attach_point;
@@ -1345,9 +1480,11 @@ void CodegenLLVM::visit(Probe &probe)
 
       tracepoint_struct_ = "";
       for (auto &match_ : matches) {
-        printf_id_ = starting_printf_id_;
-        time_id_ = starting_time_id_;
-        join_id_ = starting_join_id_;
+        printf_id_ = starting_printf_id;
+        cat_id_ = starting_cat_id;
+        system_id_ = starting_system_id;
+        time_id_ = starting_time_id;
+        join_id_ = starting_join_id;
 
         std::string full_func_id = match_;
 
@@ -1424,28 +1561,63 @@ AllocaInst *CodegenLLVM::getMapKey(Map &map)
 {
   AllocaInst *key;
   if (map.vargs) {
-    size_t size = 0;
-    for (Expression *expr : *map.vargs)
+    // A single value as a map key (e.g., @[comm] = 0;)
+    if (map.vargs->size() == 1)
     {
-      size += expr->type.size;
-    }
-    key = b_.CreateAllocaBPF(size, map.ident + "_key");
-
-    int offset = 0;
-    for (Expression *expr : *map.vargs) {
+      Expression *expr = map.vargs->at(0);
       expr->accept(*this);
-      Value *offset_val = b_.CreateGEP(key, {b_.getInt64(0), b_.getInt64(offset)});
       if (expr->type.type == Type::string || expr->type.type == Type::usym ||
         expr->type.type == Type::inet)
-        b_.CREATE_MEMCPY(offset_val, expr_, expr->type.size, 1);
+      {
+        // The value is already in the stack; Skip copy
+        key = dyn_cast<AllocaInst>(expr_);
+      }
       else
-        // promote map key to 64-bit:
-        b_.CreateStore(b_.CreateIntCast(expr_, b_.getInt64Ty(), expr->type.is_signed), offset_val);
-      offset += expr->type.size;
+      {
+        key = b_.CreateAllocaBPF(expr->type.size, map.ident + "_key");
+        b_.CreateStore(
+            b_.CreateIntCast(expr_, b_.getInt64Ty(), expr->type.is_signed),
+            key);
+      }
+    }
+    else
+    {
+      // Two or more values as a map key (e.g, @[comm, pid] = 1;)
+      size_t size = 0;
+      for (Expression *expr : *map.vargs)
+      {
+        size += expr->type.size;
+      }
+      key = b_.CreateAllocaBPF(size, map.ident + "_key");
+
+      int offset = 0;
+      // Construct a map key in the stack
+      for (Expression *expr : *map.vargs)
+      {
+        expr->accept(*this);
+        Value *offset_val =
+            b_.CreateGEP(key, { b_.getInt64(0), b_.getInt64(offset) });
+        if (expr->type.type == Type::string || expr->type.type == Type::usym ||
+            expr->type.type == Type::inet)
+        {
+          b_.CREATE_MEMCPY(offset_val, expr_, expr->type.size, 1);
+          if (!expr->is_variable && dyn_cast<AllocaInst>(expr_))
+            b_.CreateLifetimeEnd(expr_);
+        }
+        else
+        {
+          // promote map key to 64-bit:
+          b_.CreateStore(
+              b_.CreateIntCast(expr_, b_.getInt64Ty(), expr->type.is_signed),
+              offset_val);
+        }
+        offset += expr->type.size;
+      }
     }
   }
   else
   {
+    // No map key (e.g., @ = 1;). Use 0 as a key.
     key = b_.CreateAllocaBPF(SizedType(Type::integer, 8), map.ident + "_key");
     b_.CreateStore(b_.getInt64(0), key);
   }
@@ -1469,7 +1641,11 @@ AllocaInst *CodegenLLVM::getHistMapKey(Map &map, Value *log2)
       Value *offset_val = b_.CreateGEP(key, {b_.getInt64(0), b_.getInt64(offset)});
       if (expr->type.type == Type::string || expr->type.type == Type::usym ||
         expr->type.type == Type::inet)
+      {
         b_.CREATE_MEMCPY(offset_val, expr_, expr->type.size, 1);
+        if (!expr->is_variable && dyn_cast<AllocaInst>(expr_))
+          b_.CreateLifetimeEnd(expr_);
+      }
       else
         b_.CreateStore(expr_, offset_val);
       offset += expr->type.size;
@@ -1741,7 +1917,11 @@ void CodegenLLVM::createFormatStringCall(Call &call, int &id, CallArgs &call_arg
     arg.accept(*this);
     Value *offset = b_.CreateGEP(fmt_args, {b_.getInt32(0), b_.getInt32(i)});
     if (arg.type.IsArray())
+    {
       b_.CREATE_MEMCPY(offset, expr_, arg.type.size, 1);
+      if (!arg.is_variable && dyn_cast<AllocaInst>(expr_))
+        b_.CreateLifetimeEnd(expr_);
+    }
     else
       b_.CreateStore(expr_, offset);
 
@@ -1797,7 +1977,7 @@ std::unique_ptr<BpfOrc> CodegenLLVM::compile(DebugLevel debug, std::ostream &out
     raw_os_ostream llvm_ostream(out);
     llvm_ostream << "Before optimization\n";
     llvm_ostream << "-------------------\n\n";
-    module_->print(llvm_ostream, nullptr, false, true);
+    DumpIR(llvm_ostream);
   }
 
   PM.run(*module_.get());
@@ -1809,7 +1989,7 @@ std::unique_ptr<BpfOrc> CodegenLLVM::compile(DebugLevel debug, std::ostream &out
       llvm_ostream << "\nAfter optimization\n";
       llvm_ostream << "------------------\n\n";
     }
-    module_->print(llvm_ostream, nullptr, false, true);
+    DumpIR(llvm_ostream);
   }
 
   auto bpforc = std::make_unique<BpfOrc>(targetMachine);
@@ -1817,5 +1997,15 @@ std::unique_ptr<BpfOrc> CodegenLLVM::compile(DebugLevel debug, std::ostream &out
 
   return bpforc;
 }
+
+void CodegenLLVM::DumpIR() {
+  raw_os_ostream llvm_ostream(std::cout);
+  DumpIR(llvm_ostream);
+}
+
+void CodegenLLVM::DumpIR(raw_os_ostream &out) {
+  module_->print(out, nullptr, false, true);
+}
+
 } // namespace ast
 } // namespace bpftrace
