@@ -1,13 +1,17 @@
 #include <iostream>
 
 #include "irbuilderbpf.h"
-#include "libbpf.h"
 #include "bcc_usdt.h"
 #include "arch/arch.h"
 #include "utils.h"
 
 #include <llvm/IR/Module.h>
 
+namespace libbpf {
+#undef __BPF_FUNC_MAPPER
+#include "libbpf/bpf.h"
+} // namespace libbpf
+
 namespace bpftrace {
 namespace ast {
 
@@ -151,42 +155,44 @@ CallInst *IRBuilderBPF::CreateBpfPseudoCall(Map &map)
   return CreateBpfPseudoCall(mapfd);
 }
 
-CallInst *IRBuilderBPF::CreateGetJoinMap(Value *ctx __attribute__((unused)))
+CallInst *IRBuilderBPF::createMapLookup(int mapfd, AllocaInst *key)
 {
-  Value *map_ptr = CreateBpfPseudoCall(bpftrace_.join_map_->mapfd_);
-  AllocaInst *key = CreateAllocaBPF(getInt32Ty(), "key");
-  Value *keyv = getInt32(0);
-  CreateStore(keyv, key);
+  Value *map = CreateBpfPseudoCall(mapfd);
+  // void *map_lookup_elem(struct bpf_map * map, void * key)
+  // Return: Map value or NULL
 
+  assert(key->getType()->isPointerTy());
+  assert(map->getType()->isIntegerTy());
   FunctionType *lookup_func_type = FunctionType::get(
       getInt8PtrTy(),
-      {getInt8PtrTy(), getInt8PtrTy()},
+      // bpfPseudoCall returns an int64
+      {getInt64Ty(), key->getType()},
       false);
   PointerType *lookup_func_ptr_type = PointerType::get(lookup_func_type, 0);
   Constant *lookup_func = ConstantExpr::getCast(
       Instruction::IntToPtr,
-      getInt64(BPF_FUNC_map_lookup_elem),
+      getInt64(libbpf::BPF_FUNC_map_lookup_elem),
       lookup_func_ptr_type);
-  CallInst *call = CreateCall(lookup_func, {map_ptr, key}, "join_elem");
+  return CreateCall(lookup_func, { map, key }, "lookup_elem");
+}
+
+CallInst *IRBuilderBPF::CreateGetJoinMap(Value *ctx __attribute__((unused)))
+{
+  AllocaInst *key = CreateAllocaBPF(getInt32Ty(), "key");
+  CreateStore(getInt32(0), key);
+
+  CallInst *call = createMapLookup(bpftrace_.join_map_->mapfd_, key);
   return call;
 }
 
-Value *IRBuilderBPF::CreateMapLookupElem(Map &map, AllocaInst *key)
-{
-  Value *map_ptr = CreateBpfPseudoCall(map);
+Value *IRBuilderBPF::CreateMapLookupElem(Map &map, AllocaInst *key) {
+  int mapfd = bpftrace_.maps_[map.ident]->mapfd_;
+  return CreateMapLookupElem(mapfd, key, map.type);
+}
 
-  // void *map_lookup_elem(&map, &key)
-  // Return: Map value or NULL
-  FunctionType *lookup_func_type = FunctionType::get(
-      getInt8PtrTy(),
-      {getInt8PtrTy(), getInt8PtrTy()},
-      false);
-  PointerType *lookup_func_ptr_type = PointerType::get(lookup_func_type, 0);
-  Constant *lookup_func = ConstantExpr::getCast(
-      Instruction::IntToPtr,
-      getInt64(BPF_FUNC_map_lookup_elem),
-      lookup_func_ptr_type);
-  CallInst *call = CreateCall(lookup_func, {map_ptr, key}, "lookup_elem");
+Value *IRBuilderBPF::CreateMapLookupElem(int mapfd, AllocaInst *key, SizedType &type)
+{
+  CallInst *call = createMapLookup(mapfd, key);
 
   // Check if result == 0
   Function *parent = GetInsertBlock()->getParent();
@@ -194,29 +200,39 @@ Value *IRBuilderBPF::CreateMapLookupElem(Map &map, AllocaInst *key)
   BasicBlock *lookup_failure_block = BasicBlock::Create(module_.getContext(), "lookup_failure", parent);
   BasicBlock *lookup_merge_block = BasicBlock::Create(module_.getContext(), "lookup_merge", parent);
 
-  AllocaInst *value = CreateAllocaBPF(map.type, "lookup_elem_val");
+  AllocaInst *value = CreateAllocaBPF(type, "lookup_elem_val");
   Value *condition = CreateICmpNE(
       CreateIntCast(call, getInt8PtrTy(), true),
       ConstantExpr::getCast(Instruction::IntToPtr, getInt64(0), getInt8PtrTy()),
       "map_lookup_cond");
   CreateCondBr(condition, lookup_success_block, lookup_failure_block);
 
+  bool is_array = (type.type == Type::string ||
+                   (type.type == Type::cast && !type.is_pointer) ||
+                   type.type == Type::inet);
+
   SetInsertPoint(lookup_success_block);
-  if (map.type.type == Type::string || (map.type.type == Type::cast && !map.type.is_pointer) || map.type.type == Type::inet)
-    CREATE_MEMCPY(value, call, map.type.size, 1);
+  if (is_array)
+    CREATE_MEMCPY(value, call, type.size, 1);
   else
-    CreateStore(CreateLoad(getInt64Ty(), call), value);
+  {
+    assert(value->getType()->isPointerTy() &&
+           (value->getType()->getElementType() == getInt64Ty()));
+    // createMapLookup  returns an u8*
+    auto *cast = CreatePointerCast(call, value->getType(), "cast");
+    CreateStore(CreateLoad(getInt64Ty(), cast), value);
+  }
   CreateBr(lookup_merge_block);
 
   SetInsertPoint(lookup_failure_block);
-  if (map.type.type == Type::string || (map.type.type == Type::cast && !map.type.is_pointer) || map.type.type == Type::inet)
-    CreateMemSet(value, getInt8(0), map.type.size, 1);
+  if (is_array)
+    CreateMemSet(value, getInt8(0), type.size, 1);
   else
     CreateStore(getInt64(0), value);
   CreateBr(lookup_merge_block);
 
   SetInsertPoint(lookup_merge_block);
-  if (map.type.type == Type::string || (map.type.type == Type::cast && !map.type.is_pointer) || map.type.type == Type::inet)
+  if (is_array)
     return value;
 
   return CreateLoad(value);
@@ -224,21 +240,26 @@ Value *IRBuilderBPF::CreateMapLookupElem(Map &map, AllocaInst *key)
 
 void IRBuilderBPF::CreateMapUpdateElem(Map &map, AllocaInst *key, Value *val)
 {
-  Value *map_ptr = CreateBpfPseudoCall(map);
+  Value *mapid = CreateBpfPseudoCall(map);
+
+  assert(key->getType()->isPointerTy());
+  assert(mapid->getType()->isIntegerTy());
+  assert(val->getType()->isPointerTy());
+
   Value *flags = getInt64(0);
 
-  // int map_update_elem(&map, &key, &value, flags)
-  // Return: 0 on success or negative error
+  // int map_update_elem(struct bpf_map * map, void *key, void * value, u64
+  // flags) Return: 0 on success or negative error
   FunctionType *update_func_type = FunctionType::get(
       getInt64Ty(),
-      {getInt8PtrTy(), getInt8PtrTy(), getInt8PtrTy(), getInt64Ty()},
+      { getInt64Ty(), key->getType(), val->getType(), getInt64Ty() },
       false);
   PointerType *update_func_ptr_type = PointerType::get(update_func_type, 0);
   Constant *update_func = ConstantExpr::getCast(
       Instruction::IntToPtr,
-      getInt64(BPF_FUNC_map_update_elem),
+      getInt64(libbpf::BPF_FUNC_map_update_elem),
       update_func_ptr_type);
-  CreateCall(update_func, {map_ptr, key, val, flags}, "update_elem");
+  CreateCall(update_func, { mapid, key, val, flags }, "update_elem");
 }
 
 void IRBuilderBPF::CreateMapDeleteElem(Map &map, AllocaInst *key)
@@ -254,7 +275,7 @@ void IRBuilderBPF::CreateMapDeleteElem(Map &map, AllocaInst *key)
   PointerType *delete_func_ptr_type = PointerType::get(delete_func_type, 0);
   Constant *delete_func = ConstantExpr::getCast(
       Instruction::IntToPtr,
-      getInt64(BPF_FUNC_map_delete_elem),
+      getInt64(libbpf::BPF_FUNC_map_delete_elem),
       delete_func_ptr_type);
   CreateCall(delete_func, {map_ptr, key}, "delete_elem");
 }
@@ -270,7 +291,7 @@ void IRBuilderBPF::CreateProbeRead(AllocaInst *dst, size_t size, Value *src)
   PointerType *proberead_func_ptr_type = PointerType::get(proberead_func_type, 0);
   Constant *proberead_func = ConstantExpr::getCast(
       Instruction::IntToPtr,
-      getInt64(BPF_FUNC_probe_read),
+      getInt64(libbpf::BPF_FUNC_probe_read),
       proberead_func_ptr_type);
   CreateCall(proberead_func, {dst, getInt64(size), src}, "probe_read");
 }
@@ -290,7 +311,7 @@ CallInst *IRBuilderBPF::CreateProbeReadStr(AllocaInst *dst, llvm::Value *size, V
   PointerType *probereadstr_func_ptr_type = PointerType::get(probereadstr_func_type, 0);
   Constant *probereadstr_func = ConstantExpr::getCast(
       Instruction::IntToPtr,
-      getInt64(BPF_FUNC_probe_read_str),
+      getInt64(libbpf::BPF_FUNC_probe_read_str),
       probereadstr_func_ptr_type);
   return CreateCall(probereadstr_func, {dst, size, src}, "probe_read_str");
 }
@@ -305,7 +326,7 @@ CallInst *IRBuilderBPF::CreateProbeReadStr(Value *dst, size_t size, Value *src)
   PointerType *probereadstr_func_ptr_type = PointerType::get(probereadstr_func_type, 0);
   Constant *probereadstr_func = ConstantExpr::getCast(
       Instruction::IntToPtr,
-      getInt64(BPF_FUNC_probe_read_str),
+      getInt64(libbpf::BPF_FUNC_probe_read_str),
       probereadstr_func_ptr_type);
   return CreateCall(probereadstr_func, {dst, getInt64(size), src}, "map_read_str");
 }
@@ -377,45 +398,51 @@ Value *IRBuilderBPF::CreateUSDTReadArgument(Value *ctx, AttachPoint *attach_poin
 }
 
 Value *IRBuilderBPF::CreateStrcmp(Value* val, std::string str, bool inverse) {
+  auto cmpAmount = strlen(str.c_str()) + 1;
+  return CreateStrncmp(val, str, cmpAmount, inverse);
+}
+
+Value *IRBuilderBPF::CreateStrncmp(Value* val, std::string str, uint64_t n, bool inverse) {
   Function *parent = GetInsertBlock()->getParent();
   BasicBlock *str_ne = BasicBlock::Create(module_.getContext(), "strcmp.false", parent);
   AllocaInst *store = CreateAllocaBPF(getInt8Ty(), "strcmp.result");
 
-  CreateStore(getInt1(inverse), store);
+  CreateStore(getInt1(!inverse), store);
 
   const char *c_str = str.c_str();
-  for (size_t i = 0; i < strlen(c_str) + 1; i++)
+  for (size_t i = 0; i < n; i++)
   {
     BasicBlock *char_eq = BasicBlock::Create(module_.getContext(), "strcmp.loop", parent);
-    AllocaInst *val_char = CreateAllocaBPF(getInt8Ty(), "strcmp.char");
     Value *ptr = CreateAdd(
         val,
         getInt64(i));
-    CreateProbeRead(val_char, 1, ptr);
-
-    Value *l = CreateLoad(getInt8Ty(), val_char);
-    CreateLifetimeEnd(store);
+    Value *l = CreateLoad(getInt8Ty(), ptr);
     Value *r = getInt8(c_str[i]);
     Value *cmp = CreateICmpNE(l, r, "strcmp.cmp");
     CreateCondBr(cmp, str_ne, char_eq);
     SetInsertPoint(char_eq);
   }
-  CreateStore(getInt1(!inverse), store);
+  CreateStore(getInt1(inverse), store);
   CreateBr(str_ne);
 
   SetInsertPoint(str_ne);
   Value *result = CreateLoad(store);
   CreateLifetimeEnd(store);
+  result = CreateIntCast(result, getInt64Ty(), false);
   return result;
 }
 
 Value *IRBuilderBPF::CreateStrcmp(Value* val1, Value* val2, bool inverse) {
+  return CreateStrncmp(val1, val2, bpftrace_.strlen_, inverse);
+}
+
+Value *IRBuilderBPF::CreateStrncmp(Value* val1, Value* val2, uint64_t n, bool inverse) {
   /*
   // This function compares each character of the two string.
   // It returns true if all are equal and false if any are different
   // strcmp(String val1, String val2)
      {
-        for (size_t i = 0; i < bpftrace_.strlen_; i++)
+        for (size_t i = 0; i < n; i++)
         {
 
           if (val1[i] != val2[i])
@@ -436,24 +463,24 @@ Value *IRBuilderBPF::CreateStrcmp(Value* val1, Value* val2, bool inverse) {
   AllocaInst *store = CreateAllocaBPF(getInt8Ty(), "strcmp.result");
   BasicBlock *done = BasicBlock::Create(module_.getContext(), "strcmp.done", parent);
 
-  CreateStore(getInt1(inverse), store);
+  CreateStore(getInt1(!inverse), store);
 
   Value *null_byte = getInt8(0);
 
-  for (size_t i = 0; i < bpftrace_.strlen_; i++)
+  AllocaInst *val_l = CreateAllocaBPF(getInt8Ty(), "strcmp.char_l");
+  AllocaInst *val_r = CreateAllocaBPF(getInt8Ty(), "strcmp.char_r");
+  for (size_t i = 0; i < n; i++)
   {
     BasicBlock *char_eq = BasicBlock::Create(module_.getContext(), "strcmp.loop", parent);
     BasicBlock *loop_null_check = BasicBlock::Create(module_.getContext(), "strcmp.loop_null_cmp", parent);
 
-    AllocaInst *val_char1 = CreateAllocaBPF(getInt8Ty(), "strcmp.char_l");
     Value *ptr1 = CreateAdd(val1, getInt64(i));
-    CreateProbeRead(val_char1, 1, ptr1);
-    Value *l = CreateLoad(getInt8Ty(), val_char1);
+    CreateProbeRead(val_l, 1, ptr1);
+    Value *l = CreateLoad(getInt8Ty(), val_l);
 
-    AllocaInst *val_char2 = CreateAllocaBPF(getInt8Ty(), "strcmp.char_r");
     Value *ptr2 = CreateAdd(val2, getInt64(i));
-    CreateProbeRead(val_char2, 1, ptr2);
-    Value *r = CreateLoad(getInt8Ty(), val_char2);
+    CreateProbeRead(val_r, 1, ptr2);
+    Value *r = CreateLoad(getInt8Ty(), val_r);
 
     Value *cmp = CreateICmpNE(l, r, "strcmp.cmp");
     CreateCondBr(cmp, str_ne, loop_null_check);
@@ -468,13 +495,16 @@ Value *IRBuilderBPF::CreateStrcmp(Value* val1, Value* val2, bool inverse) {
 
   CreateBr(done);
   SetInsertPoint(done);
-  CreateStore(getInt1(!inverse), store);
+  CreateStore(getInt1(inverse), store);
 
   CreateBr(str_ne);
   SetInsertPoint(str_ne);
 
   Value *result = CreateLoad(store);
   CreateLifetimeEnd(store);
+  CreateLifetimeEnd(val_l);
+  CreateLifetimeEnd(val_r);
+  result = CreateIntCast(result, getInt64Ty(), false);
 
   return result;
 }
@@ -487,7 +517,7 @@ CallInst *IRBuilderBPF::CreateGetNs()
   PointerType *gettime_func_ptr_type = PointerType::get(gettime_func_type, 0);
   Constant *gettime_func = ConstantExpr::getCast(
       Instruction::IntToPtr,
-      getInt64(BPF_FUNC_ktime_get_ns),
+      getInt64(libbpf::BPF_FUNC_ktime_get_ns),
       gettime_func_ptr_type);
   return CreateCall(gettime_func, {}, "get_ns");
 }
@@ -500,29 +530,23 @@ CallInst *IRBuilderBPF::CreateGetPidTgid()
   PointerType *getpidtgid_func_ptr_type = PointerType::get(getpidtgid_func_type, 0);
   Constant *getpidtgid_func = ConstantExpr::getCast(
       Instruction::IntToPtr,
-      getInt64(BPF_FUNC_get_current_pid_tgid),
+      getInt64(libbpf::BPF_FUNC_get_current_pid_tgid),
       getpidtgid_func_ptr_type);
   return CreateCall(getpidtgid_func, {}, "get_pid_tgid");
 }
 
 CallInst *IRBuilderBPF::CreateGetCurrentCgroupId()
 {
-  #ifndef HAVE_GET_CURRENT_CGROUP_ID
-    std::cerr << "BPF_FUNC_get_current_cgroup_id is not available for your kernel version" << std::endl;
-    abort();
-  #else
-    // u64 bpf_get_current_cgroup_id(void)
-    // Return: 64-bit cgroup-v2 id
-    FunctionType *getcgroupid_func_type = FunctionType::get(getInt64Ty(), false);
-    PointerType *getcgroupid_func_ptr_type = PointerType::get(getcgroupid_func_type, 0);
-    Constant *getcgroupid_func = ConstantExpr::getCast(
+  // u64 bpf_get_current_cgroup_id(void)
+  // Return: 64-bit cgroup-v2 id
+  FunctionType *getcgroupid_func_type = FunctionType::get(getInt64Ty(), false);
+  PointerType *getcgroupid_func_ptr_type = PointerType::get(
+      getcgroupid_func_type, 0);
+  Constant *getcgroupid_func = ConstantExpr::getCast(
       Instruction::IntToPtr,
-      getInt64(BPF_FUNC_get_current_cgroup_id),
+      getInt64(libbpf::BPF_FUNC_get_current_cgroup_id),
       getcgroupid_func_ptr_type);
-    return CreateCall(getcgroupid_func, {}, "get_cgroup_id");
-  #endif
-
-
+  return CreateCall(getcgroupid_func, {}, "get_cgroup_id");
 }
 
 CallInst *IRBuilderBPF::CreateGetUidGid()
@@ -533,7 +557,7 @@ CallInst *IRBuilderBPF::CreateGetUidGid()
   PointerType *getuidgid_func_ptr_type = PointerType::get(getuidgid_func_type, 0);
   Constant *getuidgid_func = ConstantExpr::getCast(
       Instruction::IntToPtr,
-      getInt64(BPF_FUNC_get_current_uid_gid),
+      getInt64(libbpf::BPF_FUNC_get_current_uid_gid),
       getuidgid_func_ptr_type);
   return CreateCall(getuidgid_func, {}, "get_uid_gid");
 }
@@ -546,7 +570,7 @@ CallInst *IRBuilderBPF::CreateGetCpuId()
   PointerType *getcpuid_func_ptr_type = PointerType::get(getcpuid_func_type, 0);
   Constant *getcpuid_func = ConstantExpr::getCast(
       Instruction::IntToPtr,
-      getInt64(BPF_FUNC_get_smp_processor_id),
+      getInt64(libbpf::BPF_FUNC_get_smp_processor_id),
       getcpuid_func_ptr_type);
   return CreateCall(getcpuid_func, {}, "get_cpu_id");
 }
@@ -559,7 +583,7 @@ CallInst *IRBuilderBPF::CreateGetCurrentTask()
   PointerType *getcurtask_func_ptr_type = PointerType::get(getcurtask_func_type, 0);
   Constant *getcurtask_func = ConstantExpr::getCast(
       Instruction::IntToPtr,
-      getInt64(BPF_FUNC_get_current_task),
+      getInt64(libbpf::BPF_FUNC_get_current_task),
       getcurtask_func_ptr_type);
   return CreateCall(getcurtask_func, {}, "get_cur_task");
 }
@@ -572,7 +596,7 @@ CallInst *IRBuilderBPF::CreateGetRandom()
   PointerType *getrandom_func_ptr_type = PointerType::get(getrandom_func_type, 0);
   Constant *getrandom_func = ConstantExpr::getCast(
       Instruction::IntToPtr,
-      getInt64(BPF_FUNC_get_prandom_u32),
+      getInt64(libbpf::BPF_FUNC_get_prandom_u32),
       getrandom_func_ptr_type);
   return CreateCall(getrandom_func, {}, "get_random");
 }
@@ -596,7 +620,7 @@ CallInst *IRBuilderBPF::CreateGetStackId(Value *ctx, bool ustack, StackType stac
   PointerType *getstackid_func_ptr_type = PointerType::get(getstackid_func_type, 0);
   Constant *getstackid_func = ConstantExpr::getCast(
       Instruction::IntToPtr,
-      getInt64(BPF_FUNC_get_stackid),
+      getInt64(libbpf::BPF_FUNC_get_stackid),
       getstackid_func_ptr_type);
   return CreateCall(getstackid_func, {ctx, map_ptr, flags_val}, "get_stackid");
 }
@@ -612,7 +636,7 @@ void IRBuilderBPF::CreateGetCurrentComm(AllocaInst *buf, size_t size)
   PointerType *getcomm_func_ptr_type = PointerType::get(getcomm_func_type, 0);
   Constant *getcomm_func = ConstantExpr::getCast(
       Instruction::IntToPtr,
-      getInt64(BPF_FUNC_get_current_comm),
+      getInt64(libbpf::BPF_FUNC_get_current_comm),
       getcomm_func_ptr_type);
   CreateCall(getcomm_func, {buf, getInt64(size)}, "get_comm");
 }
@@ -638,10 +662,39 @@ void IRBuilderBPF::CreatePerfEventOutput(Value *ctx, Value *data, size_t size)
   PointerType *perfoutput_func_ptr_type = PointerType::get(perfoutput_func_type, 0);
   Constant *perfoutput_func = ConstantExpr::getCast(
       Instruction::IntToPtr,
-      getInt64(BPF_FUNC_perf_event_output),
+      getInt64(libbpf::BPF_FUNC_perf_event_output),
       perfoutput_func_ptr_type);
   CreateCall(perfoutput_func, {ctx, map_ptr, flags_val, data, size_val}, "perf_event_output");
 }
 
+void IRBuilderBPF::CreateSignal(Value *sig)
+{
+  // int bpf_send_signal(u32 sig)
+  // Return: 0 or error
+  FunctionType *signal_func_type = FunctionType::get(
+      getInt64Ty(),
+      {getInt32Ty()},
+      false);
+  PointerType *signal_func_ptr_type = PointerType::get(signal_func_type, 0);
+  Constant *signal_func = ConstantExpr::getCast(
+      Instruction::IntToPtr,
+      getInt64(libbpf::BPF_FUNC_send_signal),
+      signal_func_ptr_type);
+  CreateCall(signal_func, {sig}, "signal");
+}
+
+void IRBuilderBPF::CreateOverrideReturn(Value *ctx, Value *rc)
+{
+  // int bpf_override_return(struct pt_regs *regs, u64 rc)
+  // Return: 0
+  FunctionType *override_func_type = FunctionType::get(
+      getInt64Ty(), { getInt8PtrTy(), getInt64Ty() }, false);
+  PointerType *override_func_ptr_type = PointerType::get(override_func_type, 0);
+  Constant *override_func = ConstantExpr::getCast(Instruction::IntToPtr,
+      getInt64(libbpf::BPF_FUNC_override_return),
+      override_func_ptr_type);
+  CreateCall(override_func, { ctx, rc }, "override");
+}
+
 } // namespace ast
 } // namespace bpftrace
