@@ -4,8 +4,9 @@
 #include <fstream>
 #include <iostream>
 #include <link.h>
-#include <linux/perf_event.h>
 #include <linux/hw_breakpoint.h>
+#include <linux/limits.h>
+#include <linux/perf_event.h>
 #include <regex>
 #include <sys/auxv.h>
 #include <sys/utsname.h>
@@ -89,11 +90,11 @@ AttachedProbe::AttachedProbe(Probe &probe, std::tuple<uint8_t *, uintptr_t> func
   switch (probe_.type)
   {
     case ProbeType::kprobe:
-      attach_kprobe();
+      attach_kprobe(safe_mode);
       break;
     case ProbeType::kretprobe:
       check_banned_kretprobes(probe_.attach_point);
-      attach_kprobe();
+      attach_kprobe(safe_mode);
       break;
     case ProbeType::uprobe:
     case ProbeType::uretprobe:
@@ -202,7 +203,9 @@ std::string AttachedProbe::eventname() const
   {
     case ProbeType::kprobe:
     case ProbeType::kretprobe:
-      return eventprefix() + sanitise(probe_.attach_point) + index_str;
+      offset_str << std::hex << offset_;
+      return eventprefix() + sanitise(probe_.attach_point) + "_" +
+             offset_str.str() + index_str;
     case ProbeType::uprobe:
     case ProbeType::uretprobe:
     case ProbeType::usdt:
@@ -225,13 +228,6 @@ std::string AttachedProbe::sanitise(const std::string &str)
   return std::regex_replace(str, std::regex("[^A-Za-z0-9_]"), "_");
 }
 
-struct symbol {
-  std::string name;
-  uint64_t    start;
-  uint64_t    size;
-  uint64_t    address;
-};
-
 static int sym_name_cb(const char *symname, uint64_t start,
                        uint64_t size, void *p)
 {
@@ -263,7 +259,8 @@ static int sym_address_cb(const char *symname, uint64_t start,
   return 0;
 }
 
-static uint64_t resolve_offset(std::string& path, std::string& symbol, uint64_t loc)
+static uint64_t
+resolve_offset(const std::string &path, const std::string &symbol, uint64_t loc)
 {
   bcc_symbol bcc_sym;
 
@@ -273,6 +270,63 @@ static uint64_t resolve_offset(std::string& path, std::string& symbol, uint64_t
   return bcc_sym.offset;
 }
 
+static void check_alignment(std::string &path,
+                            std::string &symbol,
+                            uint64_t sym_offset,
+                            uint64_t func_offset,
+                            bool safe_mode,
+                            ProbeType type)
+{
+  Disasm dasm(path);
+  AlignState aligned = dasm.is_aligned(sym_offset, func_offset);
+  std::string probe_name = probetypeName(type);
+
+  std::string tmp = path + ":" + symbol + "+" + std::to_string(func_offset);
+
+  if (AlignState::Ok == aligned)
+    return;
+
+    // If we did not allow unaligned uprobes in the
+    // compile time, force the safe mode now.
+#ifndef HAVE_UNSAFE_PROBE
+  safe_mode = true;
+#endif
+
+  switch (aligned)
+  {
+    case AlignState::NotAlign:
+      if (safe_mode)
+        throw std::runtime_error("Could not add " + probe_name +
+                                 " into middle of instruction: " + tmp);
+      else
+        std::cerr << "Unsafe " + probe_name +
+                         " in the middle of the instruction: "
+                  << tmp << std::endl;
+      break;
+
+    case AlignState::Fail:
+      if (safe_mode)
+        throw std::runtime_error("Failed to check if " + probe_name +
+                                 " is in proper place: " + tmp);
+      else
+        std::cerr << "Unchecked " + probe_name + ": " << tmp << std::endl;
+      break;
+
+    case AlignState::NotSupp:
+      if (safe_mode)
+        throw std::runtime_error("Can't check if " + probe_name +
+                                 " is in proper place (compiled without "
+                                 "(k|u)probe offset support): " +
+                                 tmp);
+      else
+        std::cerr << "Unchecked " + probe_name + " : " << tmp << std::endl;
+      break;
+
+    default:
+      throw std::runtime_error("Internal error: " + tmp);
+  }
+}
+
 void AttachedProbe::resolve_offset_uprobe(bool safe_mode)
 {
   struct bcc_symbol_option option = { };
@@ -307,6 +361,13 @@ void AttachedProbe::resolve_offset_uprobe(bool safe_mode)
       throw std::runtime_error("Could not resolve symbol: " + probe_.path + ":" + symbol);
   }
 
+  if (probe_.type == ProbeType::uretprobe && func_offset != 0) {
+    std::stringstream msg;
+    msg << "uretprobes cannot be attached at function offset. "
+        << "(address resolved to: " << symbol << "+" << func_offset << ")";
+    throw std::runtime_error(msg.str());
+  }
+
   if (func_offset >= sym.size) {
     std::stringstream ss;
     ss << sym.size;
@@ -321,46 +382,102 @@ void AttachedProbe::resolve_offset_uprobe(bool safe_mode)
   if (func_offset == 0)
     return;
 
-  Disasm dasm(probe_.path);
-  AlignState aligned = dasm.is_aligned(sym_offset, func_offset);
+  check_alignment(
+      probe_.path, symbol, sym_offset, func_offset, safe_mode, probe_.type);
+}
 
-  std::string tmp = probe_.path + ":" + symbol + "+" + std::to_string(func_offset);
+// find vmlinux file containing the given symbol information
+static std::string find_vmlinux(const struct vmlinux_location *locs,
+                                struct symbol &sym)
+{
+  struct bcc_symbol_option option = {};
+  option.use_debug_file = 0;
+  option.use_symbol_type = BCC_SYM_ALL_TYPES;
+  struct utsname buf;
 
-  if (AlignState::Ok == aligned)
-    return;
+  uname(&buf);
 
-  // If we did not allow unaligned uprobes in the
-  // compile time, force the safe mode now.
-#ifndef HAVE_UNSAFE_UPROBE
+  for (int i = 0; locs[i].path; i++)
+  {
+    if (locs[i].raw)
+      continue; // This file is for BTF. skip
+    char path[PATH_MAX + 1];
+    snprintf(path, PATH_MAX, locs[i].path, buf.release);
+    if (access(path, R_OK))
+      continue;
+    bcc_elf_foreach_sym(path, sym_name_cb, &option, &sym);
+    if (sym.start)
+    {
+      if (bt_verbose)
+        std::cout << "vmlinux: using " << path << std::endl;
+      return path;
+    }
+  }
+
+  return "";
+}
+
+void AttachedProbe::resolve_offset_kprobe(bool safe_mode)
+{
+  struct symbol sym = {};
+  std::string &symbol = probe_.attach_point;
+  uint64_t func_offset = probe_.func_offset;
+  offset_ = func_offset;
+
+#ifndef HAVE_UNSAFE_PROBE
   safe_mode = true;
 #endif
 
-  switch (aligned)
+  if (func_offset == 0)
+    return;
+
+  sym.name = symbol;
+  const struct vmlinux_location *locs = vmlinux_locs;
+  struct vmlinux_location locs_env[] = {
+    { nullptr, true },
+    { nullptr, false },
+  };
+  char *env_path = std::getenv("BPFTRACE_VMLINUX");
+  if (env_path)
   {
-    case AlignState::NotAlign:
-      if (safe_mode)
-        throw std::runtime_error("Could not add uprobe into middle of instruction: " + tmp);
-      else
-        std::cerr << "Unsafe uprobe in the middle of the instruction: " << tmp << std::endl;
-      break;
-
-     case AlignState::Fail:
-       if (safe_mode)
-         throw std::runtime_error("Failed to check if uprobe is in proper place: " + tmp);
-       else
-         std::cerr << "Unchecked uprobe: " << tmp << std::endl;
-       break;
-
-     case AlignState::NotSupp:
-       if (safe_mode)
-         throw std::runtime_error("Can't check if uprobe is in proper place (compiled without uprobe offset support): " + tmp);
-       else
-         std::cerr << "Unchecked uprobe: " << tmp << std::endl;
-       break;
-
-     default:
-       throw std::runtime_error("Internal error: " + tmp);
+    locs_env[0].path = env_path;
+    locs = locs_env;
   }
+
+  std::string path = find_vmlinux(locs, sym);
+  if (path.empty())
+  {
+    if (safe_mode)
+    {
+      std::stringstream buf;
+      buf << "Could not resolve symbol " << symbol << ".";
+      buf << " Use BPFTRACE_VMLINUX env variable to specify vmlinux path.";
+#ifdef HAVE_UNSAFE_PROBE
+      buf << " Use --unsafe to skip the userspace check.";
+#else
+      buf << " Compile bpftrace with ALLOW_UNAFE_PROBE option to force skip "
+             "the check.";
+#endif
+      throw std::runtime_error(buf.str());
+    }
+    else
+    {
+      // linux kernel checks alignment, but not the function bounds
+      if (bt_verbose)
+        std::cout << "Could not resolve symbol " << symbol
+                  << ". Skip offset checking." << std::endl;
+      return;
+    }
+  }
+
+  if (func_offset >= sym.size)
+    throw std::runtime_error("Offset outside the function bounds ('" + symbol +
+                             "' size is " + std::to_string(sym.size) + ")");
+
+  uint64_t sym_offset = resolve_offset(path, probe_.attach_point, probe_.loc);
+
+  check_alignment(
+      path, symbol, sym_offset, func_offset, safe_mode, probe_.type);
 }
 
 /**
@@ -463,63 +580,66 @@ void AttachedProbe::load_prog()
   char name[STRING_SIZE], *namep;
   unsigned log_buf_size = sizeof (log_buf);
 
-  // Redirect stderr, so we don't get error messages from BCC
-  int old_stderr = -1, new_stderr;
-  fflush(stderr);
-  if (bt_debug != DebugLevel::kNone)
-    log_level = 15;
-  else
   {
-    old_stderr = dup(2);
-    new_stderr = open("/dev/null", O_WRONLY);
-    dup2(new_stderr, 2);
-    close(new_stderr);
-  }
+    // Redirect stderr, so we don't get error messages from BCC
+    StderrSilencer silencer;
+    if (bt_debug == DebugLevel::kNone)
+      silencer.silence();
 
-   if (bt_verbose)
-    log_level = 1;
+    if (bt_debug != DebugLevel::kNone)
+      log_level = 15;
 
-  // bpf_prog_load rejects colons in the probe name
-  strncpy(name, probe_.name.c_str(), STRING_SIZE - 1);
-  namep = name;
-  if (strrchr(name, ':') != NULL)
-    namep = strrchr(name, ':') + 1;
+    if (bt_verbose)
+      log_level = 1;
 
-  for (int attempt=0; attempt<3; attempt++)
-  {
-    auto version = kernel_version(attempt);
-    if (version == 0 && attempt > 0) {
-      // Recent kernels don't check the version so we should try to call
-      // bcc_prog_load during first iteration even if we failed to determine the
-      // version. We should not do that in subsequent iterations to avoid
-      // zeroing of log_buf on systems with older kernels.
-      continue;
-    }
+    // bpf_prog_load rejects colons in the probe name
+    strncpy(name, probe_.name.c_str(), STRING_SIZE - 1);
+    namep = name;
+    if (strrchr(name, ':') != NULL)
+      namep = strrchr(name, ':') + 1;
+
+    for (int attempt = 0; attempt < 3; attempt++)
+    {
+      auto version = kernel_version(attempt);
+      if (version == 0 && attempt > 0)
+      {
+        // Recent kernels don't check the version so we should try to call
+        // bcc_prog_load during first iteration even if we failed to determine
+        // the version. We should not do that in subsequent iterations to avoid
+        // zeroing of log_buf on systems with older kernels.
+        continue;
+      }
 
 #ifdef HAVE_BCC_PROG_LOAD
-    progfd_ = bcc_prog_load(progtype(probe_.type), namep,
+      progfd_ = bcc_prog_load(progtype(probe_.type),
+                              namep,
 #else
-    progfd_ = bpf_prog_load(progtype(probe_.type), namep,
+      progfd_ = bpf_prog_load(progtype(probe_.type),
+                              namep,
 #endif
-        reinterpret_cast<struct bpf_insn*>(insns), prog_len, license,
-        version, log_level, log_buf, log_buf_size);
-    if (progfd_ >= 0)
-      break;
-  }
-
-  // Restore stderr
-  if (bt_debug == DebugLevel::kNone)
-  {
-    fflush(stderr);
-    dup2(old_stderr, 2);
-    close(old_stderr);
+                              reinterpret_cast<struct bpf_insn *>(insns),
+                              prog_len,
+                              license,
+                              version,
+                              log_level,
+                              log_buf,
+                              log_buf_size);
+      if (progfd_ >= 0)
+        break;
+    }
   }
 
   if (progfd_ < 0) {
     if (bt_verbose) {
       std::cerr << std::endl << "Error log: " << std::endl << log_buf << std::endl;
       if (errno == ENOSPC) {
-        std::cerr << "Error: No space left on device, try increasing BPFTRACE_LOG_SIZE environment variable" << std::endl;
+        std::stringstream errmsg;
+        errmsg << "Error: Failed to load program, verification log buffer "
+               << "not big enough, try increasing the BPFTRACE_LOG_SIZE "
+               << "environment variable beyond the current value of "
+               << probe_.log_size << " bytes";
+
+        throw std::runtime_error(errmsg.str());
       }
     }
     throw std::runtime_error("Error loading program: " + probe_.name + (bt_verbose ? "" : " (try -v)"));
@@ -548,10 +668,16 @@ void AttachedProbe::load_prog()
 // although not ideal.
 typedef int (*attach_probe_wrapper_signature)(int, enum bpf_probe_attach_type, const char*, const char*, uint64_t, int);
 
-void AttachedProbe::attach_kprobe()
+void AttachedProbe::attach_kprobe(bool safe_mode)
 {
-  int perf_event_fd = cast_signature<attach_probe_wrapper_signature>(&bpf_attach_kprobe)(progfd_, attachtype(probe_.type),
-      eventname().c_str(), probe_.attach_point.c_str(), 0, 0);
+  resolve_offset_kprobe(safe_mode);
+  int perf_event_fd = cast_signature<attach_probe_wrapper_signature>(
+      &bpf_attach_kprobe)(progfd_,
+                          attachtype(probe_.type),
+                          eventname().c_str(),
+                          probe_.attach_point.c_str(),
+                          offset_,
+                          0);
 
   if (perf_event_fd < 0) {
     if (probe_.orig_name != probe_.name) {
