@@ -1,5 +1,9 @@
 #pragma once
 
+#include <string>
+#include <tuple>
+#include <vector>
+
 #include "types.h"
 
 #include "libbpf.h"
@@ -12,19 +16,22 @@ bpf_prog_type progtype(ProbeType t);
 class AttachedProbe
 {
 public:
-  AttachedProbe(Probe &probe, std::tuple<uint8_t *, uintptr_t> func, bool safe_mode);
+  AttachedProbe(Probe &probe,
+                std::tuple<uint8_t *, uintptr_t> func,
+                bool safe_mode);
   AttachedProbe(Probe &probe, std::tuple<uint8_t *, uintptr_t> func, int pid);
   ~AttachedProbe();
   AttachedProbe(const AttachedProbe &) = delete;
-  AttachedProbe& operator=(const AttachedProbe &) = delete;
+  AttachedProbe &operator=(const AttachedProbe &) = delete;
 
 private:
   std::string eventprefix() const;
   std::string eventname() const;
   static std::string sanitise(const std::string &str);
+  void resolve_offset_kprobe(bool safe_mode);
   void resolve_offset_uprobe(bool safe_mode);
   void load_prog();
-  void attach_kprobe();
+  void attach_kprobe(bool safe_mode);
   void attach_uprobe(bool safe_mode);
   void attach_usdt(int pid);
   void attach_tracepoint();
@@ -32,7 +39,7 @@ private:
   void attach_interval();
   void attach_software();
   void attach_hardware();
-  void attach_watchpoint(int pid, const std::string& mode);
+  void attach_watchpoint(int pid, const std::string &mode);
 
   Probe &probe_;
   std::tuple<uint8_t *, uintptr_t> func_;
