@@ -1,11 +1,11 @@
-#include <assert.h>
+#include <arpa/inet.h>
+#include <cassert>
+#include <ctime>
 #include <fstream>
 #include <iomanip>
 #include <iostream>
 #include <sstream>
 #include <sys/epoll.h>
-#include <time.h>
-#include <arpa/inet.h>
 
 #include <fcntl.h>
 #include <sys/prctl.h>
@@ -45,7 +45,7 @@ int format(char * s, size_t n, const char * fmt, std::vector<std::unique_ptr<IPr
   int ret = -1;
   switch(args.size()) {
     case 0:
-      ret = snprintf(s, n, fmt);
+      ret = snprintf(s, n, "%s", fmt);
       break;
     case 1:
       ret = snprintf(s, n, fmt, args.at(0)->value());
@@ -80,13 +80,14 @@ int format(char * s, size_t n, const char * fmt, std::vector<std::unique_ptr<IPr
 
 BPFtrace::~BPFtrace()
 {
-  for (int pid : child_pids_)
+  kill_child();
+  if (child_pid() != 0)
   {
     // We don't care if waitpid returns any errors. We're just trying
     // to make a best effort here. It's not like we could recover from
     // an error.
     int status;
-    waitpid(pid, &status, 0);
+    waitpid(child_pid(), &status, 0);
   }
 
   for (const auto& pair : exe_sym_)
@@ -322,15 +323,30 @@ int BPFtrace::num_probes() const
   return special_probes_.size() + probes_.size();
 }
 
+void BPFtrace::kill_child()
+{
+  if (child_pid() == 0)
+    return;
+
+  if (child_running_) {
+    kill(child_pid(), SIGTERM);
+  } else {
+#pragma GCC diagnostic push
+#pragma GCC diagnostic ignored "-Wunused-result"
+    // We need to disable this warning for some GCC/libc combinations despite
+    // using the void cast: https://gcc.gnu.org/bugzilla/show_bug.cgi?id=66425
+    (void)write(child_start_pipe_, &CHILD_EXIT_QUIETLY, 1);
+#pragma GCC diagnostic pop
+    close(child_start_pipe_);
+  }
+}
+
 void BPFtrace::request_finalize()
 {
   finalize_ = true;
   attached_probes_.clear();
+  kill_child();
 
-  for (int pid : child_pids_)
-  {
-    kill(pid, SIGTERM);
-  }
 }
 
 void perf_event_printer(void *cb_cookie, void *data, int size __attribute__((unused)))
@@ -360,8 +376,8 @@ void perf_event_printer(void *cb_cookie, void *data, int size __attribute__((unu
   else if (printf_id == asyncactionint(AsyncAction::print))
   {
     std::string arg = (const char *)(static_cast<uint8_t*>(data) + sizeof(uint64_t) + 2 * sizeof(uint64_t));
-    uint64_t top = (uint64_t)*(static_cast<uint64_t*>(data) + sizeof(uint64_t) / sizeof(uint64_t));
-    uint64_t div = (uint64_t)*(static_cast<uint64_t*>(data) + (sizeof(uint64_t) + sizeof(uint64_t)) / sizeof(uint64_t));
+    uint64_t top = (uint64_t) * (static_cast<uint64_t *>(data) + 1);
+    uint64_t div = (uint64_t) * (static_cast<uint64_t *>(data) + 2);
     err = bpftrace->print_map_ident(arg, top, div);
     if (err)
       throw std::runtime_error("Could not print map with ident \"" + arg + "\", err=" + std::to_string(err));
@@ -394,7 +410,7 @@ void perf_event_printer(void *cb_cookie, void *data, int size __attribute__((unu
       std::cerr << "localtime: " << strerror(errno) << std::endl;
       return;
     }
-    uint64_t time_id = (uint64_t)*(static_cast<uint64_t*>(data) + sizeof(uint64_t) / sizeof(uint64_t));
+    uint64_t time_id = (uint64_t) * (static_cast<uint64_t *>(data) + 1);
     auto fmt = bpftrace->time_args_[time_id].c_str();
     if (strftime(timestr, sizeof(timestr), fmt, tmp) == 0) {
       std::cerr << "strftime returned 0" << std::endl;
@@ -405,7 +421,7 @@ void perf_event_printer(void *cb_cookie, void *data, int size __attribute__((unu
   }
   else if (printf_id == asyncactionint(AsyncAction::join))
   {
-    uint64_t join_id = (uint64_t)*(static_cast<uint64_t*>(data) + sizeof(uint64_t) / sizeof(uint64_t));
+    uint64_t join_id = (uint64_t) * (static_cast<uint64_t *>(data) + 1);
     auto delim = bpftrace->join_args_[join_id].c_str();
     std::stringstream joined;
     for (unsigned int i = 0; i < bpftrace->join_argnum_; i++) {
@@ -594,19 +610,6 @@ std::vector<std::unique_ptr<IPrintable>> BPFtrace::get_arg_values(const std::vec
   return arg_values;
 }
 
-bool BPFtrace::is_numeric(std::string str) const
-{
-  int i = 0;
-  while (str[i]) {
-    if (str[i] < '0' || str[i] > '9')
-      return false;
-    i++;
-  }
-  if (i == 0)
-    return false;
-  return true;
-}
-
 void BPFtrace::add_param(const std::string &param)
 {
   params_.emplace_back(param);
@@ -687,8 +690,6 @@ bool attach_reverse(const Probe &p)
 
 int BPFtrace::run(std::unique_ptr<BpfOrc> bpforc)
 {
-  int wait_for_tracing_pipe;
-
   auto r_special_probes = special_probes_.rbegin();
   for (; r_special_probes != special_probes_.rend(); ++r_special_probes)
   {
@@ -702,26 +703,18 @@ int BPFtrace::run(std::unique_ptr<BpfOrc> bpforc)
   if (epollfd < 0)
     return epollfd;
 
-  // Spawn a child process if we've been passed a command to run
-  if (cmd_.size())
+  if (elapsed_map_)
   {
-    auto args = split_string(cmd_, ' ');
-    auto paths = resolve_binary_path(args[0]); // does path lookup on executable
-    if (paths.size() > 1)
+    struct timespec ts;
+    clock_gettime(CLOCK_MONOTONIC, &ts);
+    auto nsec = 1000000000ULL * ts.tv_sec + ts.tv_nsec;
+    uint64_t key = 0;
+
+    if (bpf_update_elem(elapsed_map_->mapfd_, &key, &nsec, 0) < 0)
     {
-      std::cerr << "path '" << args[0] << "' must refer to a unique binary but matched " << paths.size() << std::endl;
+      perror("Failed to write start time to elapsed map");
       return -1;
     }
-    args[0] = paths.front().c_str();
-    int pid = spawn_child(args, &wait_for_tracing_pipe);
-    if (pid < 0)
-    {
-      std::cerr << "Failed to spawn child=" << cmd_ << std::endl;
-      return pid;
-    }
-
-    child_pids_.emplace_back(pid);
-    pid_ = pid;
   }
 
   BEGIN_trigger();
@@ -738,7 +731,7 @@ int BPFtrace::run(std::unique_ptr<BpfOrc> bpforc)
       auto attached_probe = attach_probe(*probes, *bpforc.get());
       if (attached_probe == nullptr)
       {
-        write(wait_for_tracing_pipe, &CHILD_EXIT_QUIETLY, 1);
+        kill_child();
         return -1;
       }
       attached_probes_.push_back(std::move(attached_probe));
@@ -751,7 +744,7 @@ int BPFtrace::run(std::unique_ptr<BpfOrc> bpforc)
       auto attached_probe = attach_probe(*r_probes, *bpforc.get());
       if (attached_probe == nullptr)
       {
-        write(wait_for_tracing_pipe, &CHILD_EXIT_QUIETLY, 1);
+        kill_child();
         return -1;
       }
       attached_probes_.push_back(std::move(attached_probe));
@@ -759,16 +752,17 @@ int BPFtrace::run(std::unique_ptr<BpfOrc> bpforc)
   }
 
   // Kick the child to execute the command.
-  if (cmd_.size())
+  if (has_child_cmd())
   {
-    int ret = write(wait_for_tracing_pipe, &CHILD_GO, 1);
+    int ret = write(child_start_pipe_, &CHILD_GO, 1);
     if (ret < 0)
     {
       perror("unable to write to 'go' pipe");
       return ret;
     }
 
-    close(wait_for_tracing_pipe);
+    child_running_ = true;
+    close(child_start_pipe_);
   }
 
   if (bt_verbose)
@@ -981,6 +975,7 @@ int BPFtrace::clear_map(IMap &map)
 // zero a map
 int BPFtrace::zero_map(IMap &map)
 {
+  uint32_t nvalues = map.is_per_cpu_type() ? ncpus_ : 1;
   std::vector<uint8_t> old_key;
   try
   {
@@ -1007,12 +1002,7 @@ int BPFtrace::zero_map(IMap &map)
     old_key = key;
   }
 
-  int value_size = map.type_.size;
-  if (map.type_.type == Type::count || map.type_.type == Type::sum ||
-      map.type_.type == Type::min || map.type_.type == Type::max ||
-      map.type_.type == Type::avg || map.type_.type == Type::hist ||
-      map.type_.type == Type::lhist || map.type_.type == Type::stats )
-    value_size *= ncpus_;
+  int value_size = map.type_.size * nvalues;
   std::vector<uint8_t> zero(value_size, 0);
   for (auto &key : keys)
   {
@@ -1030,40 +1020,46 @@ int BPFtrace::zero_map(IMap &map)
 
 std::string BPFtrace::map_value_to_str(IMap &map, std::vector<uint8_t> value, uint32_t div)
 {
+  uint32_t nvalues = map.is_per_cpu_type() ? ncpus_ : 1;
   if (map.type_.type == Type::kstack)
-    return get_stack(*(uint64_t*)value.data(), false, map.type_.stack_type, 8);
+    return get_stack(
+        read_data<uint64_t>(value.data()), false, map.type_.stack_type, 8);
   else if (map.type_.type == Type::ustack)
-    return get_stack(*(uint64_t*)value.data(), true, map.type_.stack_type, 8);
+    return get_stack(
+        read_data<uint64_t>(value.data()), true, map.type_.stack_type, 8);
   else if (map.type_.type == Type::ksym)
-    return resolve_ksym(*(uintptr_t*)value.data());
+    return resolve_ksym(read_data<uintptr_t>(value.data()));
   else if (map.type_.type == Type::usym)
-    return resolve_usym(*(uintptr_t*)value.data(), *(uint64_t*)(value.data() + 8));
+    return resolve_usym(read_data<uintptr_t>(value.data()),
+                        read_data<uintptr_t>(value.data() + 8));
   else if (map.type_.type == Type::inet)
-    return resolve_inet(*(int32_t*)value.data(), (uint8_t*)(value.data() + 8));
+    return resolve_inet(read_data<uint32_t>(value.data()),
+                        (uint8_t *)(value.data() + 8));
   else if (map.type_.type == Type::username)
-    return resolve_uid(*(uint64_t*)(value.data()));
+    return resolve_uid(read_data<uint64_t>(value.data()));
   else if (map.type_.type == Type::string)
     return std::string(reinterpret_cast<const char*>(value.data()));
   else if (map.type_.type == Type::count)
-    return std::to_string(reduce_value<uint64_t>(value, ncpus_) / div);
+    return std::to_string(reduce_value<uint64_t>(value, nvalues) / div);
   else if (map.type_.type == Type::sum || map.type_.type == Type::integer) {
     if (map.type_.is_signed)
-      return std::to_string(reduce_value<int64_t>(value, ncpus_) / div);
+      return std::to_string(reduce_value<int64_t>(value, nvalues) / div);
 
-    return std::to_string(reduce_value<uint64_t>(value, ncpus_) / div);
+    return std::to_string(reduce_value<uint64_t>(value, nvalues) / div);
   }
   else if (map.type_.type == Type::min)
-    return std::to_string(min_value(value, ncpus_) / div);
+    return std::to_string(min_value(value, nvalues) / div);
   else if (map.type_.type == Type::max)
-    return std::to_string(max_value(value, ncpus_) / div);
+    return std::to_string(max_value(value, nvalues) / div);
   else if (map.type_.type == Type::probe)
-    return resolve_probe(*(uint64_t*)value.data());
+    return resolve_probe(read_data<uint64_t>(value.data()));
   else
-    return std::to_string(*(int64_t*)value.data() / div);
+    return std::to_string(read_data<int64_t>(value.data()) / div);
 }
 
 int BPFtrace::print_map(IMap &map, uint32_t top, uint32_t div)
 {
+  uint32_t nvalues = map.is_per_cpu_type() ? ncpus_ : 1;
   std::vector<uint8_t> old_key;
   try
   {
@@ -1082,9 +1078,7 @@ int BPFtrace::print_map(IMap &map, uint32_t top, uint32_t div)
   while (bpf_get_next_key(map.mapfd_, old_key.data(), key.data()) == 0)
   {
     int value_size = map.type_.size;
-    if (map.type_.type == Type::count || map.type_.type == Type::sum ||
-        map.type_.type == Type::min || map.type_.type == Type::max || map.type_.type == Type::integer)
-      value_size *= ncpus_;
+    value_size *= nvalues;
     auto value = std::vector<uint8_t>(value_size);
     int err = bpf_lookup_elem(map.mapfd_, key.data(), value.data());
     if (err == -1)
@@ -1110,22 +1104,22 @@ int BPFtrace::print_map(IMap &map, uint32_t top, uint32_t div)
     std::sort(values_by_key.begin(), values_by_key.end(), [&](auto &a, auto &b)
     {
       if (is_signed)
-        return reduce_value<int64_t>(a.second, ncpus_) < reduce_value<int64_t>(b.second, ncpus_);
-      return reduce_value<uint64_t>(a.second, ncpus_) < reduce_value<uint64_t>(b.second, ncpus_);
+        return reduce_value<int64_t>(a.second, nvalues) < reduce_value<int64_t>(b.second, nvalues);
+      return reduce_value<uint64_t>(a.second, nvalues) < reduce_value<uint64_t>(b.second, nvalues);
     });
   }
   else if (map.type_.type == Type::min)
   {
     std::sort(values_by_key.begin(), values_by_key.end(), [&](auto &a, auto &b)
     {
-      return min_value(a.second, ncpus_) < min_value(b.second, ncpus_);
+      return min_value(a.second, nvalues) < min_value(b.second, nvalues);
     });
   }
   else if (map.type_.type == Type::max)
   {
     std::sort(values_by_key.begin(), values_by_key.end(), [&](auto &a, auto &b)
     {
-      return max_value(a.second, ncpus_) < max_value(b.second, ncpus_);
+      return max_value(a.second, nvalues) < max_value(b.second, nvalues);
     });
   }
   else
@@ -1146,6 +1140,7 @@ int BPFtrace::print_map_hist(IMap &map, uint32_t top, uint32_t div)
   // e.g. A map defined as: @x[1, 2] = @hist(3);
   // would actually be stored with the key: [1, 2, 3]
 
+  uint32_t nvalues = map.is_per_cpu_type() ? ncpus_ : 1;
   std::vector<uint8_t> old_key;
   try
   {
@@ -1169,7 +1164,7 @@ int BPFtrace::print_map_hist(IMap &map, uint32_t top, uint32_t div)
     for (size_t i=0; i<map.key_.size(); i++)
       key_prefix.at(i) = key.at(i);
 
-    int value_size = map.type_.size * ncpus_;
+    int value_size = map.type_.size * nvalues;
     auto value = std::vector<uint8_t>(value_size);
     int err = bpf_lookup_elem(map.mapfd_, key.data(), value.data());
     if (err == -1)
@@ -1192,7 +1187,7 @@ int BPFtrace::print_map_hist(IMap &map, uint32_t top, uint32_t div)
       else
         values_by_key[key_prefix] = std::vector<uint64_t>(1002);
     }
-    values_by_key[key_prefix].at(bucket) = reduce_value<uint64_t>(value, ncpus_);
+    values_by_key[key_prefix].at(bucket) = reduce_value<uint64_t>(value, nvalues);
 
     old_key = key;
   }
@@ -1221,6 +1216,7 @@ int BPFtrace::print_map_hist(IMap &map, uint32_t top, uint32_t div)
 
 int BPFtrace::print_map_stats(IMap &map)
 {
+  uint32_t nvalues = map.is_per_cpu_type() ? ncpus_ : 1;
   // stats() and avg() maps add an extra 8 bytes onto the end of their key for
   // storing the bucket number.
 
@@ -1247,7 +1243,7 @@ int BPFtrace::print_map_stats(IMap &map)
     for (size_t i=0; i<map.key_.size(); i++)
       key_prefix.at(i) = key.at(i);
 
-    int value_size = map.type_.size * ncpus_;
+    int value_size = map.type_.size * nvalues;
     auto value = std::vector<uint8_t>(value_size);
     int err = bpf_lookup_elem(map.mapfd_, key.data(), value.data());
     if (err == -1)
@@ -1267,7 +1263,7 @@ int BPFtrace::print_map_stats(IMap &map)
       // New key - create a list of buckets for it
       values_by_key[key_prefix] = std::vector<int64_t>(2);
     }
-    values_by_key[key_prefix].at(bucket) = reduce_value<int64_t>(value, ncpus_);
+    values_by_key[key_prefix].at(bucket) = reduce_value<int64_t>(value, nvalues);
 
     old_key = key;
   }
@@ -1295,23 +1291,40 @@ int BPFtrace::print_map_stats(IMap &map)
   return 0;
 }
 
-int BPFtrace::spawn_child(const std::vector<std::string>& args, int *notify_trace_start_pipe_fd)
+pid_t BPFtrace::spawn_child()
 {
   static const int maxargs = 256;
   char* argv[maxargs];
   int wait_for_tracing_pipe[2];
 
+  auto args = split_string(cmd_, ' ');
+  auto paths = resolve_binary_path(args[0]); // does path lookup on executable
+  switch (paths.size())
+  {
+  case 0:
+    std::cerr << "path '" << args[0] << "' does not exist or is not executable" << std::endl;
+    return -1;
+  case 1:
+    args[0] = paths.front().c_str();
+    break;
+  default:
+    std::cerr << "path '" << args[0] << "' must refer to a unique binary but matched "
+              << paths.size() << " binaries" << std::endl;
+    return -1;
+  }
+
+
+  if (args.size() >= (maxargs - 1))
+  {
+    std::cerr << "Too many args passed into spawn_child (" << args.size()
+              << " > " << maxargs - 1 << ")" << std::endl;
+    return -1;
+  }
+
   // Convert vector of strings into raw array of C-strings for execve(2)
   int idx = 0;
   for (const auto& arg : args)
   {
-    if (idx == maxargs - 1)
-    {
-      std::cerr << "Too many args passed into spawn_child (" << args.size()
-        << " > " << maxargs - 1 << ")" << std::endl;
-      return -1;
-    }
-
     argv[idx] = const_cast<char*>(arg.c_str());
     ++idx;
   }
@@ -1324,8 +1337,8 @@ int BPFtrace::spawn_child(const std::vector<std::string>& args, int *notify_trac
   }
 
   // Fork and exec
-  int ret = fork();
-  if (ret == 0)
+  pid_t pid = fork();
+  if (pid == 0)
   {
     // Receive SIGTERM if parent dies
     //
@@ -1341,7 +1354,7 @@ int BPFtrace::spawn_child(const std::vector<std::string>& args, int *notify_trac
 
     char bf;
 
-    ret = read(wait_for_tracing_pipe[0], &bf, 1);
+    int ret = read(wait_for_tracing_pipe[0], &bf, 1);
     if (ret != 1)
     {
       perror("failed to read 'go' pipe");
@@ -1356,18 +1369,22 @@ int BPFtrace::spawn_child(const std::vector<std::string>& args, int *notify_trac
 
     if (execve(argv[0], argv, environ))
     {
-      perror("execve");
+      auto err = "Failed to execve: " + std::string(argv[0]);
+      perror(err.c_str());
       return -1;
     }
   }
-  else if (ret > 0)
+  else if (pid > 0)
   {
-    *notify_trace_start_pipe_fd = wait_for_tracing_pipe[1];
-    return ret;
+    close(wait_for_tracing_pipe[0]);
+    child_start_pipe_ = wait_for_tracing_pipe[1];
+    child_pid_ = pid;
+    pid_ = pid;
+    return pid;
   }
   else
   {
-    perror("fork");
+    perror("Failed to fork");
     return -1;
   }
 
@@ -1375,34 +1392,34 @@ int BPFtrace::spawn_child(const std::vector<std::string>& args, int *notify_trac
 }
 
 template <typename T>
-T BPFtrace::reduce_value(const std::vector<uint8_t> &value, int ncpus)
+T BPFtrace::reduce_value(const std::vector<uint8_t> &value, int nvalues)
 {
   T sum = 0;
-  for (int i=0; i<ncpus; i++)
+  for (int i=0; i<nvalues; i++)
   {
-    sum += *(const T*)(value.data() + i*sizeof(T*));
+    sum += read_data<T>(value.data() + i * sizeof(T));
   }
   return sum;
 }
 
-uint64_t BPFtrace::max_value(const std::vector<uint8_t> &value, int ncpus)
+uint64_t BPFtrace::max_value(const std::vector<uint8_t> &value, int nvalues)
 {
   uint64_t val, max = 0;
-  for (int i=0; i<ncpus; i++)
+  for (int i=0; i<nvalues; i++)
   {
-    val = *(const uint64_t*)(value.data() + i*sizeof(uint64_t*));
+    val = read_data<uint64_t>(value.data() + i * sizeof(uint64_t));
     if (val > max)
       max = val;
   }
   return max;
 }
 
-int64_t BPFtrace::min_value(const std::vector<uint8_t> &value, int ncpus)
+int64_t BPFtrace::min_value(const std::vector<uint8_t> &value, int nvalues)
 {
   int64_t val, max = 0, retval;
-  for (int i=0; i<ncpus; i++)
+  for (int i=0; i<nvalues; i++)
   {
-    val = *(const int64_t*)(value.data() + i*sizeof(int64_t*));
+    val = read_data<int64_t>(value.data() + i * sizeof(int64_t));
     if (val > max)
       max = val;
   }
@@ -1427,12 +1444,8 @@ std::vector<uint8_t> BPFtrace::find_empty_key(IMap &map, size_t size) const
 {
   if (size == 0) size = 8;
   auto key = std::vector<uint8_t>(size);
-  int value_size = map.type_.size;
-  if (map.type_.type == Type::count || map.type_.type == Type::hist ||
-      map.type_.type == Type::sum || map.type_.type == Type::min ||
-      map.type_.type == Type::max || map.type_.type == Type::avg ||
-      map.type_.type == Type::stats || map.type_.type == Type::lhist)
-    value_size *= ncpus_;
+  uint32_t nvalues = map.is_per_cpu_type() ? ncpus_ : 1;
+  int value_size = map.type_.size * nvalues;
   auto value = std::vector<uint8_t>(value_size);
 
   if (bpf_lookup_elem(map.mapfd_, key.data(), value.data()))
@@ -1583,38 +1596,44 @@ uint64_t BPFtrace::resolve_cgroupid(const std::string &path) const
 }
 
 #ifdef HAVE_BCC_ELF_FOREACH_SYM
-static int sym_resolve_callback(const char *name, uint64_t addr, uint64_t, void *payload)
+static int sym_resolve_callback(const char *name,
+                                uint64_t addr,
+                                uint64_t size,
+                                void *payload)
 {
-  struct bcc_symbol *sym = (struct bcc_symbol *)payload;
-  if (!strcmp(name, sym->name)) {
-    sym->offset = addr;
+  struct symbol *sym = (struct symbol *)payload;
+  if (!strcmp(name, sym->name.c_str()))
+  {
+    sym->address = addr;
+    sym->size = size;
     return -1;
   }
   return 0;
 }
 #endif
 
-uint64_t BPFtrace::resolve_uname(const std::string &name, const std::string &path) const
+int BPFtrace::resolve_uname(const std::string &name,
+                            struct symbol *sym,
+                            const std::string &path) const
 {
+  sym->name = name;
 #ifdef HAVE_BCC_ELF_FOREACH_SYM
   struct bcc_symbol_option option;
   memset(&option, 0, sizeof(option));
   option.use_symbol_type = (1 << STT_OBJECT);
 
-  struct bcc_symbol sym;
-  sym.name = name.c_str();
-  sym.offset = 0;
-
-  int err = bcc_elf_foreach_sym(path.c_str(), sym_resolve_callback, &option, &sym);
-  if (err == -1 || (err == 0 && sym.offset == 0))
-    throw std::runtime_error("Could not resolve symbol: " + path + ":" + name);
-
-  return sym.offset;
+  return bcc_elf_foreach_sym(path.c_str(), sym_resolve_callback, &option, sym);
 #else
-  std::string call_str = std::string("objdump -tT ") + path + " | grep -w " + name;
+  std::string call_str =
+      std::string("objdump -tT ") + path + " | grep -w " + sym->name;
   const char *call = call_str.c_str();
   auto result = exec_system(call);
-  return read_address_from_output(result);
+  sym->address = read_address_from_output(result);
+  /* Trying to grab the size from objdump output is not that easy. foreaech_sym
+     has been around for a while, users should switch to that.
+  */
+  sym->size = 8;
+  return 0;
 #endif
 }
 
@@ -1655,7 +1674,6 @@ uint64_t BPFtrace::read_address_from_output(std::string output)
   return std::stoull(first_word, 0, 16);
 }
 
-
 static std::string resolve_inetv4(const uint8_t* inet) {
   char addr_cstr[INET_ADDRSTRLEN];
   inet_ntop(AF_INET, inet, addr_cstr, INET_ADDRSTRLEN);
@@ -1763,17 +1781,21 @@ void BPFtrace::sort_by_key(std::vector<SizedType> key_args,
     {
       if (arg.size == 8)
       {
-        std::stable_sort(values_by_key.begin(), values_by_key.end(), [&](auto &a, auto &b)
-        {
-          return *(const uint64_t*)(a.first.data() + arg_offset) < *(const uint64_t*)(b.first.data() + arg_offset);
-        });
+        std::stable_sort(
+            values_by_key.begin(), values_by_key.end(), [&](auto &a, auto &b) {
+              auto va = read_data<uint64_t>(a.first.data() + arg_offset);
+              auto vb = read_data<uint64_t>(b.first.data() + arg_offset);
+              return va < vb;
+            });
       }
       else if (arg.size == 4)
       {
-        std::stable_sort(values_by_key.begin(), values_by_key.end(), [&](auto &a, auto &b)
-        {
-          return *(const uint32_t*)(a.first.data() + arg_offset) < *(const uint32_t*)(b.first.data() + arg_offset);
-        });
+        std::stable_sort(
+            values_by_key.begin(), values_by_key.end(), [&](auto &a, auto &b) {
+              auto va = read_data<uint32_t>(a.first.data() + arg_offset);
+              auto vb = read_data<uint32_t>(b.first.data() + arg_offset);
+              return va < vb;
+            });
       }
       else
       {
@@ -1858,13 +1880,13 @@ void BPFtrace::log_with_location(std::string level, std::ostream &out, const loc
 
   // print only the message if location info wasn't set
   if (l.begin.line == 0) {
-    out << level << ": " << m << std::endl;
+    out << level << ": " << msg << std::endl;
     return;
   }
 
   if (l.begin.line > l.end.line) {
     out << "BUG: begin > end: " << l.begin << ":" << l.end << std::endl;
-    out << level << ": " << m << std::endl;
+    out << level << ": " << msg << std::endl;
     return;
   }
 
@@ -1872,7 +1894,7 @@ void BPFtrace::log_with_location(std::string level, std::ostream &out, const loc
      <filename>:<start_line>-<end_line>: ERROR: <message>
   */
   if (l.begin.line < l.end.line) {
-    out << l.begin.line << "-" << l.end.line << ": ERROR: " << m << std::endl;
+    out << l.begin.line << "-" << l.end.line << ": ERROR: " << msg << std::endl;
     return;
   }
 
@@ -1890,7 +1912,7 @@ void BPFtrace::log_with_location(std::string level, std::ostream &out, const loc
             ~~~~~~~~~~
   */
   out << l.begin.line << ":" << l.begin.column << "-" << l.end.column;
-  out << ": " << level << ": " << m << std::endl;
+  out << ": " << level << ": " << msg << std::endl;
   std::string srcline = get_source_line(l.begin.line - 1);
 
   if (srcline == "")
@@ -1905,8 +1927,12 @@ void BPFtrace::log_with_location(std::string level, std::ostream &out, const loc
   }
   out << std::endl;
 
-  for (unsigned int x = 0; x < srcline.size() && x < (l.end.column - 1); x++) {
-    char marker = (x < (l.begin.column - 1)) ? ' ' : '~';
+  for (unsigned int x = 0;
+       x < srcline.size() && x < (static_cast<unsigned int>(l.end.column) - 1);
+       x++)
+  {
+    char marker = (x < (static_cast<unsigned int>(l.begin.column) - 1)) ? ' '
+                                                                        : '~';
     if (srcline[x] == '\t') {
       out << std::string(4, marker);
     } else {
