@@ -4,21 +4,30 @@
 #include <map>
 #include <memory>
 #include <set>
-#include <vector>
 #include <unordered_map>
 #include <utility>
+#include <vector>
 
 #include "ast.h"
 #include "attached_probe.h"
+#include "btf.h"
 #include "imap.h"
+#include "output.h"
 #include "printf.h"
 #include "struct.h"
-#include "utils.h"
 #include "types.h"
-#include "output.h"
+#include "utils.h"
 
 namespace bpftrace {
 
+struct symbol
+{
+  std::string name;
+  uint64_t start;
+  uint64_t size;
+  uint64_t address;
+};
+
 class BpfOrc;
 enum class DebugLevel;
 
@@ -26,15 +35,17 @@ enum class DebugLevel;
 extern DebugLevel bt_debug;
 extern bool bt_verbose;
 
-enum class DebugLevel {
+enum class DebugLevel
+{
   kNone,
   kDebug,
   kFullDebug
 };
 
-inline DebugLevel operator++(DebugLevel& level, int)
+inline DebugLevel operator++(DebugLevel &level, int)
 {
-  switch (level) {
+  switch (level)
+  {
     case DebugLevel::kNone:
       level = DebugLevel::kDebug;
       break;
@@ -91,20 +102,25 @@ public:
   std::string resolve_inet(int af, const uint8_t* inet) const;
   std::string resolve_uid(uintptr_t addr) const;
   uint64_t resolve_kname(const std::string &name) const;
-  uint64_t resolve_uname(const std::string &name, const std::string &path) const;
+  virtual int resolve_uname(const std::string &name,
+                            struct symbol *sym,
+                            const std::string &path) const;
   std::string map_value_to_str(IMap &map, std::vector<uint8_t> value, uint32_t div);
   virtual std::string extract_func_symbols_from_path(const std::string &path) const;
   std::string resolve_probe(uint64_t probe_id) const;
   uint64_t resolve_cgroupid(const std::string &path) const;
   std::vector<std::unique_ptr<IPrintable>> get_arg_values(const std::vector<Field> &args, uint8_t* arg_data);
   void add_param(const std::string &param);
-  bool is_numeric(std::string str) const;
   std::string get_param(size_t index, bool is_str) const;
   size_t num_params() const;
   void request_finalize();
   void error(std::ostream &out, const location &l, const std::string &m);
   void warning(std::ostream &out, const location &l, const std::string &m);
   void log_with_location(std::string, std::ostream &, const location &, const std::string &);
+  bool has_child_cmd() { return cmd_.size() != 0; }
+  virtual pid_t child_pid() { return child_pid_; };
+  int spawn_child();
+  void kill_child();
 
   std::string cmd_;
   int pid_{0};
@@ -123,6 +139,7 @@ public:
   std::vector<std::tuple<std::string, std::vector<Field>>> cat_args_;
   std::unordered_map<StackType, std::unique_ptr<IMap>> stackid_maps_;
   std::unique_ptr<IMap> join_map_;
+  std::unique_ptr<IMap> elapsed_map_;
   std::unique_ptr<IMap> perf_event_map_;
   std::vector<std::string> probe_ids_;
   unsigned int join_argnum_;
@@ -141,20 +158,24 @@ public:
 
   static void sort_by_key(
       std::vector<SizedType> key_args,
-      std::vector<std::pair<std::vector<uint8_t>,
-      std::vector<uint8_t>>> &values_by_key);
+      std::vector<std::pair<std::vector<uint8_t>, std::vector<uint8_t>>>
+          &values_by_key);
   std::set<std::string> find_wildcard_matches(
       const ast::AttachPoint &attach_point) const;
   std::set<std::string> find_wildcard_matches(
       const std::string &prefix,
       const std::string &func,
       std::istream &symbol_stream) const;
-  virtual std::unique_ptr<std::istream> get_symbols_from_file(const std::string &path) const;
+  virtual std::unique_ptr<std::istream> get_symbols_from_file(
+      const std::string &path) const;
   virtual std::unique_ptr<std::istream> get_symbols_from_usdt(
       int pid,
       const std::string &target) const;
   const std::string get_source_line(unsigned int);
 
+  BTF btf_;
+  std::unordered_set<std::string> btf_set_;
+
 protected:
   std::vector<Probe> probes_;
   std::vector<Probe> special_probes_;
@@ -166,17 +187,22 @@ private:
   std::map<std::string, std::pair<int, void *>> exe_sym_; // exe -> (pid, cache)
   int ncpus_;
   int online_cpus_;
-  std::vector<int> child_pids_;
   std::vector<std::string> params_;
   int next_probe_id_ = 0;
 
+  pid_t child_pid_ = 0;
+  bool child_running_ = false; // true when `CHILD_GO` has been sent (child
+                               // execve)
+  int child_start_pipe_ = -1;
+
   std::string src_;
   std::string filename_;
   std::vector<std::string> srclines_;
 
-  std::unique_ptr<AttachedProbe> attach_probe(Probe &probe, const BpfOrc &bpforc);
+  std::unique_ptr<AttachedProbe> attach_probe(Probe &probe,
+                                              const BpfOrc &bpforc);
   int setup_perf_events();
-  void poll_perf_events(int epollfd, bool drain=false);
+  void poll_perf_events(int epollfd, bool drain = false);
   int clear_map(IMap &map);
   int zero_map(IMap &map);
   int print_map(IMap &map, uint32_t top, uint32_t div);
@@ -184,13 +210,16 @@ private:
   int print_map_lhist(IMap &map);
   int print_map_stats(IMap &map);
   int print_hist(const std::vector<uint64_t> &values, uint32_t div) const;
-  int print_lhist(const std::vector<uint64_t> &values, int min, int max, int step) const;
-  template <typename T> static T reduce_value(const std::vector<uint8_t> &value, int ncpus);
-  static int64_t min_value(const std::vector<uint8_t> &value, int ncpus);
-  static uint64_t max_value(const std::vector<uint8_t> &value, int ncpus);
+  int print_lhist(const std::vector<uint64_t> &values,
+                  int min,
+                  int max,
+                  int step) const;
+  template <typename T>
+  static T reduce_value(const std::vector<uint8_t> &value, int nvalues);
+  static int64_t min_value(const std::vector<uint8_t> &value, int nvalues);
+  static uint64_t max_value(const std::vector<uint8_t> &value, int nvalues);
   static uint64_t read_address_from_output(std::string output);
   std::vector<uint8_t> find_empty_key(IMap &map, size_t size) const;
-  static int spawn_child(const std::vector<std::string>& args, int *notify_trace_start_pipe_fd);
   static bool is_pid_alive(int pid);
 };
 
