@@ -1,28 +1,28 @@
-#include <iostream>
-#include <sys/types.h>
-#include <sys/stat.h>
-#include <fcntl.h>
-#include <unistd.h>
-#include <sys/utsname.h>
-#include <string.h>
-#include <linux/limits.h>
 #include "btf.h"
-#include "types.h"
 #include "bpftrace.h"
+#include "types.h"
+#include "utils.h"
+#include <cstring>
+#include <fcntl.h>
+#include <iostream>
+#include <linux/limits.h>
+#include <regex>
+#include <sys/stat.h>
+#include <sys/types.h>
+#include <sys/utsname.h>
+#include <unistd.h>
 
 #ifdef HAVE_LIBBPF_BTF_DUMP
 #include <linux/bpf.h>
 #include <linux/btf.h>
+#pragma GCC diagnostic push
+#pragma GCC diagnostic ignored "-Wcast-qual"
 #include <bpf/btf.h>
+#pragma GCC diagnostic pop
 #include <bpf/libbpf.h>
 
 namespace bpftrace {
 
-struct btf_location {
-  const char *path; // path with possible "%s" format to be replaced current release
-  bool        raw;  // file is either as ELF (false) or raw BTF data (true)
-};
-
 static unsigned char *get_data(const char *file, ssize_t *sizep)
 {
   struct stat st;
@@ -87,7 +87,7 @@ static int libbpf_print(enum libbpf_print_level level, const char *msg, va_list
   return vfprintf(stderr, msg, ap);
 }
 
-static struct btf *btf_open(struct btf_location *locs)
+static struct btf *btf_open(const struct vmlinux_location *locs)
 {
   struct utsname buf;
 
@@ -134,36 +134,20 @@ static struct btf *btf_open(struct btf_location *locs)
 
 BTF::BTF(void) : btf(nullptr), state(NODATA)
 {
-  // 'borrowed' from libbpf's bpf_core_find_kernel_btf
-  // from Andrii Nakryiko
-  struct btf_location locs_normal[] =
-  {
-    { "/sys/kernel/btf/vmlinux",                 true  },
-    { "/boot/vmlinux-%1$s",                      false },
-    { "/lib/modules/%1$s/vmlinux-%1$s",          false },
-    { "/lib/modules/%1$s/build/vmlinux",         false },
-    { "/usr/lib/modules/%1$s/kernel/vmlinux",    false },
-    { "/usr/lib/debug/boot/vmlinux-%1$s",        false },
-    { "/usr/lib/debug/boot/vmlinux-%1$s.debug",  false },
-    { "/usr/lib/debug/lib/modules/%1$s/vmlinux", false },
+  struct vmlinux_location locs_env[] = {
+    { nullptr, true },
     { nullptr, false },
   };
 
-  struct btf_location locs_test[] =
-  {
-    { nullptr, true  },
-    { nullptr, false },
-  };
+  const struct vmlinux_location *locs = vmlinux_locs;
 
-  struct btf_location *locs = locs_normal;
-
-  // Try to get BTF file from BPFTRACE_BTF_TEST env
-  char *path = std::getenv("BPFTRACE_BTF_TEST");
+  // Try to get BTF file from BPFTRACE_BTF env
+  char *path = std::getenv("BPFTRACE_BTF");
 
   if (path)
   {
-    locs_test[0].path = path;
-    locs = locs_test;
+    locs_env[0].path = path;
+    locs = locs_env;
   }
 
   btf = btf_open(locs);
@@ -172,7 +156,7 @@ BTF::BTF(void) : btf(nullptr), state(NODATA)
     libbpf_set_print(libbpf_print);
     state = OK;
   }
-  else
+  else if (bt_debug != DebugLevel::kNone)
   {
     std::cerr << "BTF: failed to find BTF data " << std::endl;
   }
@@ -203,6 +187,24 @@ static const char *btf_str(const struct btf *btf, __u32 off)
   return btf__name_by_offset(btf, off) ? : "(invalid)";
 }
 
+static std::string full_type_str(const struct btf *btf, const struct btf_type *type)
+{
+  const char *str = btf_str(btf, type->name_off);
+
+  if (BTF_INFO_KIND(type->info) == BTF_KIND_STRUCT)
+    return std::string("struct ") + str;
+
+  if (BTF_INFO_KIND(type->info) == BTF_KIND_UNION)
+    return std::string("union ") + str;
+
+  return str;
+}
+
+static std::string btf_type_str(const std::string& type)
+{
+  return std::regex_replace(type, std::regex("^(struct )|(union )"), "");
+}
+
 std::string BTF::c_def(std::unordered_set<std::string>& set)
 {
   std::string ret = std::string("");
@@ -226,56 +228,13 @@ std::string BTF::c_def(std::unordered_set<std::string>& set)
   for (id = 1; id <= max && myset.size(); id++)
   {
     const struct btf_type *t = btf__type_by_id(btf, id);
-    const char *str = btf_str(btf, t->name_off);
+    std::string str = full_type_str(btf, t);
 
-    /*
-     * TODO(danobi): Only resolve necessary types. This can be
-     * accomlished by doing something like:
-     *
-     *    for (...) {
-     *      const struct btf_type *t = ...
-     *      const char *str = ...
-     *
-     *      if (myset.find(str) != myset.end()) {
-     *        btf_dump__dump_type(...);
-     *        myset.erase(str);
-     *
-     *        <dump required forward declared types>
-     *      }
-     *    }
-     *
-     * Care are must be taken to fully resolve any accesses
-     * via a pointer. libbpf currently does not fully resolve types
-     * that are used as a pointer. For example, if in the kernel there
-     * was:
-     *
-     *    struct Bar {
-     *      int x;
-     *    };
-     *
-     *    struct Foo {
-     *      struct Bar *b;
-     *    };
-     *
-     * b would not be fully resolved. Instead, libbpf will dump a
-     * forward declaration. So the libbpf output would look like:
-     *
-     *    struct Bar;
-     *    struct Foo {
-     *      struct Bar *b;
-     *    }
-     *
-     * So for now, we read in every type BTF exposes. It's slower
-     * and less memory efficient, but it's more ergonomic for the
-     * user to not have to do an explicit cast every time they
-     * access a pointer.
-     */
-
-    for (id = 1; id <= max; id++)
+    auto it = myset.find(str);
+    if (it != myset.end())
     {
-      const struct btf_type *t = btf__type_by_id(btf, id);
-      const char *str = btf_str(btf, t->name_off);
       btf_dump__dump_type(dump, id);
+      myset.erase(it);
     }
   }
 
@@ -283,6 +242,63 @@ std::string BTF::c_def(std::unordered_set<std::string>& set)
   return ret;
 }
 
+std::string BTF::type_of(const std::string& name, const std::string& field)
+{
+  __s32 type_id = btf__find_by_name(btf, btf_type_str(name).c_str());
+
+  if (type_id < 0)
+    return std::string("");
+
+  const struct btf_type *type = btf__type_by_id(btf, type_id);
+  return type_of(type, field);
+}
+
+std::string BTF::type_of(const btf_type *type, const std::string &field)
+{
+  if (!type ||
+      (BTF_INFO_KIND(type->info) != BTF_KIND_STRUCT &&
+       BTF_INFO_KIND(type->info) != BTF_KIND_UNION))
+    return std::string("");
+
+  // We need to walk through oaa the struct/union members
+  // and try to find the requested field name.
+  //
+  // More info on struct/union members:
+  //  https://www.kernel.org/doc/html/latest/bpf/btf.html#btf-kind-union
+  const struct btf_member *m = reinterpret_cast<const struct btf_member*>(type + 1);
+
+  for (unsigned int i = 0; i < BTF_INFO_VLEN(type->info); i++)
+  {
+    std::string m_name = btf__name_by_offset(btf, m[i].name_off);
+
+    // anonymous struct/union
+    if (m_name == "")
+    {
+      const struct btf_type *type = btf__type_by_id(btf, m[i].type);
+      std::string type_name = type_of(type, field);
+      if (!type_name.empty())
+        return type_name;
+    }
+
+    if (m_name != field)
+      continue;
+
+    const struct btf_type *f = btf__type_by_id(btf, m[i].type);
+
+    if (!f)
+      break;
+
+    // Get rid of all the pointers on the way to the actual type.
+    while (BTF_INFO_KIND(f->info) == BTF_KIND_PTR) {
+      f = btf__type_by_id(btf, f->type);
+    }
+
+    return full_type_str(btf, f);
+  }
+
+  return std::string("");
+}
+
 } // namespace bpftrace
 
 #else // HAVE_LIBBPF_BTF_DUMP
@@ -295,6 +311,11 @@ BTF::~BTF() { }
 
 std::string BTF::c_def(std::unordered_set<std::string>& set __attribute__((__unused__))) { return std::string(""); }
 
+std::string BTF::type_of(const std::string& name __attribute__((__unused__)),
+                         const std::string& field __attribute__((__unused__))) {
+  return std::string("");
+}
+
 } // namespace bpftrace
 
 #endif // HAVE_LIBBPF_BTF_DUMP
