@@ -5,12 +5,14 @@
 #include <unordered_set>
 
 struct btf;
+struct btf_type;
 
 namespace bpftrace {
 
 class BTF
 {
-  enum state {
+  enum state
+  {
     NODATA,
     OK,
   };
@@ -21,9 +23,11 @@ public:
 
   bool has_data(void);
   std::string c_def(std::unordered_set<std::string>& set);
+  std::string type_of(const std::string& name, const std::string& field);
+  std::string type_of(const btf_type* type, const std::string& field);
 
 private:
-  struct btf *btf;
+  struct btf* btf;
   enum state state = NODATA;
 };
 
