@@ -1,18 +1,23 @@
 #pragma once
 
-#include <clang-c/Index.h>
 #include "bpftrace.h"
+#include <clang-c/Index.h>
 
-#define ASM_GOTO_WORKAROUND_H "asm_goto_workaround.h"
+#define CLANG_WORKAROUNDS_H "clang_workarounds.h"
 
 namespace bpftrace {
 
-namespace ast { class Program; }
+namespace ast {
+class Program;
+}
 
 class ClangParser
 {
 public:
-  bool parse(ast::Program *program, BPFtrace &bpftrace, std::vector<std::string> extra_flags = {});
+  bool parse(ast::Program *program,
+             BPFtrace &bpftrace,
+             std::vector<std::string> extra_flags = {});
+
 private:
   bool visit_children(CXCursor &cursor, BPFtrace &bpftrace);
   bool parse_btf_definitions(BPFtrace &bpftrace);
@@ -27,13 +32,13 @@ private:
     CXTranslationUnit get_translation_unit();
 
     CXErrorCode parse_translation_unit(const char *source_filename,
-                            const char *const *command_line_args,
-                            int num_command_line_args,
-                            struct CXUnsavedFile *unsaved_files,
-                            unsigned num_unsaved_files,
-                            unsigned options);
+                                       const char *const *command_line_args,
+                                       int num_command_line_args,
+                                       struct CXUnsavedFile *unsaved_files,
+                                       unsigned num_unsaved_files,
+                                       unsigned options);
 
-    bool check_diagnostics(const std::string& input);
+    bool check_diagnostics(const std::string &input);
 
     CXCursor get_translation_unit_cursor();
 
