@@ -7,9 +7,10 @@
 
 namespace bpftrace {
 
-class Field;
+struct Field;
 
-std::string verify_format_string(const std::string &fmt, std::vector<Field> args);
+std::string verify_format_string(const std::string& fmt,
+                                 std::vector<Field> args);
 
 class IPrintable
 {
@@ -23,6 +24,7 @@ class PrintableString : public virtual IPrintable
 public:
   PrintableString(std::string value) : value_(std::move(value)) { }
   uint64_t value();
+
 private:
   std::string value_;
 };
@@ -32,6 +34,7 @@ class PrintableCString : public virtual IPrintable
 public:
   PrintableCString(char* value) : value_(value) { }
   uint64_t value();
+
 private:
   char* value_;
 };
@@ -41,9 +44,9 @@ class PrintableInt : public virtual IPrintable
 public:
   PrintableInt(uint64_t value) : value_(value) { }
   uint64_t value();
+
 private:
   uint64_t value_;
 };
 
-
 } // namespace bpftrace
