@@ -58,21 +58,28 @@ struct StackType
   }
 };
 
-class SizedType
+struct SizedType
 {
-public:
   SizedType() : type(Type::none), size(0) { }
- SizedType(Type type, size_t size_, bool is_signed, const std::string &cast_type = "")
-    : type(type), size(size_), is_signed(is_signed), cast_type(cast_type) { }
+  SizedType(Type type,
+            size_t size_,
+            bool is_signed,
+            const std::string &cast_type = "")
+      : type(type), size(size_), is_signed(is_signed), cast_type(cast_type)
+  {
+  }
   SizedType(Type type, size_t size_, const std::string &cast_type = "")
-    : type(type), size(size_), cast_type(cast_type) { }
+      : type(type), size(size_), cast_type(cast_type)
+  {
+  }
 
-  SizedType(Type type, StackType stack_type_)
-    : SizedType(type, 8) {
+  SizedType(Type type, StackType stack_type_) : SizedType(type, 8)
+  {
     stack_type = stack_type_;
   }
   Type type;
-  Type elem_type = Type::none; // Array element type if accessing elements of an array
+  Type elem_type = Type::none; // Array element type if accessing elements of an
+                               // array
   size_t size;
   StackType stack_type;
   bool is_signed = false;
@@ -137,9 +144,8 @@ ProbeType probetype(const std::string &type);
 std::string probetypeName(const std::string &type);
 std::string probetypeName(ProbeType t);
 
-class Probe
+struct Probe
 {
-public:
   ProbeType type;
   std::string path;             // file path if used
   std::string attach_point;     // probe name (last component)
@@ -184,14 +190,14 @@ enum class PositionalParameterType
 
 } // namespace bpftrace
 
-
 namespace std {
-template<>
+template <>
 struct hash<bpftrace::StackType>
 {
-  size_t operator()(const bpftrace::StackType& obj) const
+  size_t operator()(const bpftrace::StackType &obj) const
   {
-    switch (obj.mode) {
+    switch (obj.mode)
+    {
       case bpftrace::StackMode::bpftrace:
         return std::hash<std::string>()("bpftrace#" + to_string(obj.limit));
       case bpftrace::StackMode::perf:
@@ -202,4 +208,4 @@ struct hash<bpftrace::StackType>
     }
   }
 };
-}
+} // namespace std
