@@ -10,6 +10,14 @@ namespace parser {
 
 using Printer = ast::Printer;
 
+void test_parse_failure(const std::string &input)
+{
+  BPFtrace bpftrace;
+  std::stringstream out;
+  Driver driver(bpftrace, out);
+  ASSERT_EQ(driver.parse_str(input), 1);
+}
+
 void test(const std::string &input, const std::string &output)
 {
   BPFtrace bpftrace;
@@ -714,10 +722,21 @@ TEST(Parser, call)
 
 TEST(Parser, call_unknown_function)
 {
-  test("kprobe:sys_open { myfunc() }",
-      "Program\n"
-      " kprobe:sys_open\n"
-      "  call: myfunc\n");
+  test_parse_failure("kprobe:sys_open { myfunc() }");
+  test_parse_failure("k:f { probe(); }");
+}
+
+TEST(Parser, call_builtin)
+{
+  // Builtins should not be usable as function
+  test_parse_failure("k:f { nsecs(); }");
+  test_parse_failure("k:f { nsecs  (); }");
+  test_parse_failure("k:f { nsecs(\"abc\"); }");
+  test_parse_failure("k:f { nsecs(123); }");
+
+  test_parse_failure("k:f { probe(\"blah\"); }");
+  test_parse_failure("k:f { probe(); }");
+  test_parse_failure("k:f { probe(123); }");
 }
 
 TEST(Parser, call_kaddr)
@@ -787,10 +806,10 @@ TEST(Parser, usdt_namespaced_probe)
 
 TEST(Parser, escape_chars)
 {
-  test("kprobe:sys_open { \"newline\\nand tab\\tbackslash\\\\quote\\\"here\" }",
+  test("kprobe:sys_open { \"newline\\nand tab\\tcr\\rbackslash\\\\quote\\\"here oct\\1009hex\\x309\" }",
       "Program\n"
       " kprobe:sys_open\n"
-      "  string: newline\\nand tab\\tbackslash\\\\quote\\\"here\n");
+      "  string: newline\\nand tab\\tcr\\rbackslash\\\\quote\\\"here oct@9hex09\n");
 }
 
 TEST(Parser, begin_probe)
@@ -929,6 +948,52 @@ TEST(Parser, wildcard_path)
       "Program\n"
       " usdt:/my/program*foo:func\n"
       "  int: 1\n");
+  // Make sure calls or builtins don't cause issues
+  test("usdt:/my/program*avg:func { 1; }",
+       "Program\n"
+       " usdt:/my/program*avg:func\n"
+       "  int: 1\n");
+  test("usdt:/my/program*nsecs:func { 1; }",
+       "Program\n"
+       " usdt:/my/program*nsecs:func\n"
+       "  int: 1\n");
+}
+
+TEST(Parser, dot_in_func)
+{
+  test("uprobe:/my/go/program:runtime.main.func1 { 1; }",
+       "Program\n"
+       " uprobe:/my/go/program:runtime.main.func1\n"
+       "  int: 1\n");
+}
+
+TEST(Parser, wildcard_func)
+{
+  test("usdt:/my/program:abc*cd { 1; }",
+       "Program\n"
+       " usdt:/my/program:abc*cd\n"
+       "  int: 1\n");
+  test("usdt:/my/program:abc*c*d { 1; }",
+       "Program\n"
+       " usdt:/my/program:abc*c*d\n"
+       "  int: 1\n");
+
+  std::string keywords[] = {
+    "arg0", "args", "curtask", "func", "gid" "rand", "uid",
+    "avg", "cat", "exit", "kaddr", "min", "printf", "usym",
+    "kstack", "ustack", "bpftrace", "perf", "uprobe", "kprobe",
+  };
+  for(auto kw : keywords)
+  {
+    test("usdt:/my/program:"+ kw +"*c*d { 1; }",
+         "Program\n"
+         " usdt:/my/program:"+ kw + "*c*d\n"
+         "  int: 1\n");
+    test("usdt:/my/program:abc*"+ kw +"*c*d { 1; }",
+         "Program\n"
+         " usdt:/my/program:abc*"+ kw + "*c*d\n"
+         "  int: 1\n");
+  }
 }
 
 TEST(Parser, short_map_name)
@@ -990,6 +1055,34 @@ TEST(Parser, brackets)
 }
 
 TEST(Parser, cast)
+{
+  test("kprobe:sys_read { (struct mytype)arg0; }",
+      "Program\n"
+      " kprobe:sys_read\n"
+      "  (struct mytype)\n"
+      "   builtin: arg0\n");
+  test("kprobe:sys_read { (union mytype)arg0; }",
+      "Program\n"
+      " kprobe:sys_read\n"
+      "  (union mytype)\n"
+      "   builtin: arg0\n");
+}
+
+TEST(Parser, cast_ptr)
+{
+  test("kprobe:sys_read { (struct mytype*)arg0; }",
+      "Program\n"
+      " kprobe:sys_read\n"
+      "  (struct mytype*)\n"
+      "   builtin: arg0\n");
+  test("kprobe:sys_read { (union mytype*)arg0; }",
+      "Program\n"
+      " kprobe:sys_read\n"
+      "  (union mytype*)\n"
+      "   builtin: arg0\n");
+}
+
+TEST(Parser, cast_typedef)
 {
   test("kprobe:sys_read { (mytype)arg0; }",
       "Program\n"
@@ -998,7 +1091,7 @@ TEST(Parser, cast)
       "   builtin: arg0\n");
 }
 
-TEST(Parser, cast_ptr)
+TEST(Parser, cast_ptr_typedef)
 {
   test("kprobe:sys_read { (mytype*)arg0; }",
       "Program\n"
@@ -1211,6 +1304,33 @@ kprobe:f { "asdf }
   EXPECT_EQ(out.str(), expected);
 }
 
+TEST(Parser, uprobe_offset)
+{
+  test("u:./test:fn+1 {}",
+       "Program\n"
+       " uprobe:./test:fn+1\n");
+  test("u:./test:fn+0x10 {}",
+       "Program\n"
+       " uprobe:./test:fn+16\n");
+
+  test("u:./test:\"fn.abc\"+1 {}",
+       "Program\n"
+       " uprobe:./test:fn.abc+1\n");
+  test("u:./test:\"fn.abc\"+0x10 {}",
+       "Program\n"
+       " uprobe:./test:fn.abc+16\n");
+}
+
+TEST(Parser, invalid_increment_decrement)
+{
+  test_parse_failure("i:s:1 { @=5++}");
+  test_parse_failure("i:s:1 { @=++5}");
+  test_parse_failure("i:s:1 { @=5--}");
+  test_parse_failure("i:s:1 { @=--5}");
+  test_parse_failure("i:s:1 { @=\"a\"++}");
+}
+
+
 } // namespace parser
 } // namespace test
 } // namespace bpftrace
