@@ -28,7 +28,8 @@ void gen_bytecode(const std::string &input, std::stringstream &out)
   ClangParser clang;
   clang.parse(driver.root_, *bpftrace);
 
-  ast::SemanticAnalyser semantics(driver.root_, *bpftrace);
+  MockBPFfeature feature = MockBPFfeature();
+  ast::SemanticAnalyser semantics(driver.root_, *bpftrace, feature);
   ASSERT_EQ(semantics.analyse(), 0);
   ASSERT_EQ(semantics.create_maps(true), 0);
 
