@@ -1,10 +1,11 @@
-#include "gmock/gmock.h"
-#include "gtest/gtest.h"
+#include "semantic_analyser.h"
+#include "bpffeature.h"
 #include "bpftrace.h"
 #include "clang_parser.h"
 #include "driver.h"
 #include "mocks.h"
-#include "semantic_analyser.h"
+#include "gmock/gmock.h"
+#include "gtest/gtest.h"
 
 namespace bpftrace {
 namespace test {
@@ -28,8 +29,9 @@ void test_for_warning(
   clang.parse(driver.root_, bpftrace);
 
   ASSERT_EQ(driver.parse_str(input), 0);
+  MockBPFfeature feature;
   std::stringstream out;
-  ast::SemanticAnalyser semantics(driver.root_, bpftrace, out);
+  ast::SemanticAnalyser semantics(driver.root_, bpftrace, feature, out);
   semantics.analyse();
   if (invert)
     EXPECT_THAT(out.str(), Not(HasSubstr(warning)));
@@ -47,13 +49,12 @@ void test_for_warning(
   test_for_warning(*bpftrace, input, warning, invert, safe_mode);
 }
 
-
-void test(
-    BPFtrace &bpftrace,
-    Driver &driver,
-    const std::string &input,
-    int expected_result=0,
-    bool safe_mode = true)
+void test(BPFtrace &bpftrace,
+          BPFfeature &feature,
+          Driver &driver,
+          const std::string &input,
+          int expected_result = 0,
+          bool safe_mode = true)
 {
   bpftrace.safe_mode_ = safe_mode;
   ASSERT_EQ(driver.parse_str(input), 0);
@@ -63,7 +64,7 @@ void test(
 
   ASSERT_EQ(driver.parse_str(input), 0);
   std::stringstream out;
-  ast::SemanticAnalyser semantics(driver.root_, bpftrace, out);
+  ast::SemanticAnalyser semantics(driver.root_, bpftrace, feature, out);
   std::stringstream msg;
   msg << "\nInput:\n" << input << "\n\nOutput:\n";
   EXPECT_EQ(expected_result, semantics.analyse()) << msg.str() + out.str();
@@ -75,7 +76,8 @@ void test(BPFtrace &bpftrace,
     bool safe_mode = true)
 {
   Driver driver(bpftrace);
-  test(bpftrace, driver, input, expected_result, safe_mode);
+  MockBPFfeature feature = MockBPFfeature();
+  test(bpftrace, feature, driver, input, expected_result, safe_mode);
 }
 
 void test(Driver &driver,
@@ -84,16 +86,26 @@ void test(Driver &driver,
     bool safe_mode = true)
 {
   auto bpftrace = get_mock_bpftrace();
-  test(*bpftrace, driver, input, expected_result, safe_mode);
+  MockBPFfeature feature = MockBPFfeature();
+  test(*bpftrace, feature, driver, input, expected_result, safe_mode);
+}
+
+void test(BPFfeature &feature,
+          const std::string &input,
+          int expected_result = 0,
+          bool safe_mode = true)
+{
+  auto bpftrace = get_mock_bpftrace();
+  Driver driver(*bpftrace);
+  test(*bpftrace, feature, driver, input, expected_result, safe_mode);
 }
 
 void test(const std::string &input,
     int expected_result=0,
     bool safe_mode = true)
 {
-  auto bpftrace = get_mock_bpftrace();
-  Driver driver(*bpftrace);
-  test(*bpftrace, driver, input, expected_result, safe_mode);
+  MockBPFfeature feature = MockBPFfeature();
+  test(feature, input, expected_result, safe_mode);
 }
 
 TEST(semantic_analyser, builtin_variables)
@@ -101,9 +113,7 @@ TEST(semantic_analyser, builtin_variables)
   // Just check that each builtin variable exists.
   test("kprobe:f { pid }", 0);
   test("kprobe:f { tid }", 0);
-  #ifdef HAVE_GET_CURRENT_CGROUP_ID
-    test("kprobe:f { cgroup }", 0);
-  #endif
+  test("kprobe:f { cgroup }", 0);
   test("kprobe:f { uid }", 0);
   test("kprobe:f { username }", 0);
   test("kprobe:f { gid }", 0);
@@ -121,9 +131,25 @@ TEST(semantic_analyser, builtin_variables)
   test("kprobe:f { sarg0 }", 0);
   test("kretprobe:f { retval }", 0);
   test("kprobe:f { func }", 0);
+  test("uprobe:/bin/sh:f { func }", 0);
   test("kprobe:f { probe }", 0);
   test("tracepoint:a:b { args }", 0);
   test("kprobe:f { fake }", 1);
+
+  MockBPFfeature feature(false);
+  test(feature, "k:f { cgroup }", 1);
+}
+
+TEST(semantic_analyser, builtin_cpid)
+{
+  auto bpftrace = get_mock_bpftrace();
+  test(*bpftrace, "i:ms:100 { printf(\"%d\\n\", cpid); }", 1, false);
+  test(*bpftrace, "i:ms:100 { @=cpid }", 1, false);
+  test(*bpftrace, "i:ms:100 { $a=cpid }", 1, false);
+  bpftrace->cmd_ = "sleep 1";
+  test(*bpftrace, "i:ms:100 { printf(\"%d\\n\", cpid); }", 0, false);
+  test(*bpftrace, "i:ms:100 { @=cpid }", 0, false);
+  test(*bpftrace, "i:ms:100 { $a=cpid }", 0, false);
 }
 
 TEST(semantic_analyser, builtin_functions)
@@ -152,14 +178,13 @@ TEST(semantic_analyser, builtin_functions)
   test("kprobe:f { ksym(0xffff) }", 0);
   test("kprobe:f { usym(0xffff) }", 0);
   test("kprobe:f { kaddr(\"sym\") }", 0);
-  test("kprobe:f { uaddr(\"sym\") }", 0);
   test("kprobe:f { ntop(0xffff) }", 0);
   test("kprobe:f { ntop(2, 0xffff) }", 0);
   test("kprobe:f { reg(\"ip\") }", 0);
   test("kprobe:f { kstack(1) }", 0);
   test("kprobe:f { ustack(1) }", 0);
-  test("kprobe:f { fake() }", 1);
   test("kprobe:f { cat(\"/proc/uptime\") }", 0);
+  test("uprobe:/bin/bash:main { uaddr(\"glob_asciirange\") }", 0);
 }
 
 TEST(semantic_analyser, undefined_map)
@@ -436,6 +461,8 @@ TEST(semantic_analyser, call_time)
   test("kprobe:f { @x = time(); }", 1);
   test("kprobe:f { $x = time(); }", 1);
   test("kprobe:f { @[time()] = 1; }", 1);
+  test("kprobe:f { time(1); }", 10);
+  test("kprobe:f { $x = \"str\"; time($x); }", 10);
 }
 
 TEST(semantic_analyser, call_str)
@@ -486,22 +513,22 @@ TEST(semantic_analyser, call_ntop)
 
   test("kprobe:f { ntop(2, arg0); }", 0);
   test("kprobe:f { ntop(arg0); }", 0);
-  test(structs + "kprobe:f { ntop(10, ((inet*)0)->ipv4); }", 0);
-  test(structs + "kprobe:f { ntop(10, ((inet*)0)->ipv6); }", 0);
-  test(structs + "kprobe:f { ntop(((inet*)0)->ipv4); }", 0);
-  test(structs + "kprobe:f { ntop(((inet*)0)->ipv6); }", 0);
+  test(structs + "kprobe:f { ntop(10, ((struct inet*)0)->ipv4); }", 0);
+  test(structs + "kprobe:f { ntop(10, ((struct inet*)0)->ipv6); }", 0);
+  test(structs + "kprobe:f { ntop(((struct inet*)0)->ipv4); }", 0);
+  test(structs + "kprobe:f { ntop(((struct inet*)0)->ipv6); }", 0);
 
   test("kprobe:f { @x = ntop(2, arg0); }", 0);
   test("kprobe:f { @x = ntop(arg0); }", 0);
   test("kprobe:f { @x = ntop(2, 0xFFFF); }", 0);
   test("kprobe:f { @x = ntop(0xFFFF); }", 0);
-  test(structs + "kprobe:f { @x = ntop(((inet*)0)->ipv4); }", 0);
-  test(structs + "kprobe:f { @x = ntop(((inet*)0)->ipv6); }", 0);
+  test(structs + "kprobe:f { @x = ntop(((struct inet*)0)->ipv4); }", 0);
+  test(structs + "kprobe:f { @x = ntop(((struct inet*)0)->ipv6); }", 0);
 
   test("kprobe:f { ntop(); }", 1);
   test("kprobe:f { ntop(2, \"hello\"); }", 1);
   test("kprobe:f { ntop(\"hello\"); }", 1);
-  test(structs + "kprobe:f { ntop(((inet*)0)->invalid); }", 1);
+  test(structs + "kprobe:f { ntop(((struct inet*)0)->invalid); }", 1);
 }
 
 TEST(semantic_analyser, call_kaddr)
@@ -514,12 +541,44 @@ TEST(semantic_analyser, call_kaddr)
 
 TEST(semantic_analyser, call_uaddr)
 {
-  test("kprobe:f { uaddr(\"counter\"); }", 0);
-  test("kprobe:f { uaddr(\"github.com/golang/glog.severityName\"); }", 0);
-  test("kprobe:f { @x = uaddr(\"counter\"); }", 0);
-  test("kprobe:f { uaddr(); }", 1);
-  test("kprobe:f { uaddr(123); }", 1);
-  test("kprobe:f { uaddr(\"?\"); }", 1);
+  test("u:/bin/bash:main { uaddr(\"github.com/golang/glog.severityName\"); }", 0);
+  test("uprobe:/bin/bash:main { uaddr(\"glob_asciirange\"); }", 0);
+  test("u:/bin/bash:main,u:/bin/bash:readline { uaddr(\"glob_asciirange\"); }",
+       0);
+  test("uprobe:/bin/bash:main { @x = uaddr(\"glob_asciirange\"); }", 0);
+  test("uprobe:/bin/bash:main { uaddr(); }", 1);
+  test("uprobe:/bin/bash:main { uaddr(123); }", 1);
+  test("uprobe:/bin/bash:main { uaddr(\"?\"); }", 1);
+  test("uprobe:/bin/bash:main { $str = \"glob_asciirange\"; uaddr($str); }", 1);
+  test("uprobe:/bin/bash:main { @str = \"glob_asciirange\"; uaddr(@str); }", 1);
+
+  test("k:f { uaddr(\"A\"); }", 1);
+  test("i:s:1 { uaddr(\"A\"); }", 1);
+
+  // The C struct parser should set the is_signed flag on signed types
+  BPFtrace bpftrace;
+  Driver driver(bpftrace);
+  std::string prog = "uprobe:/bin/bash:main {"
+                     "$a = uaddr(\"12345_1\");"
+                     "$b = uaddr(\"12345_2\");"
+                     "$c = uaddr(\"12345_4\");"
+                     "$d = uaddr(\"12345_8\");"
+                     "$e = uaddr(\"12345_5\");"
+                     "$f = uaddr(\"12345_33\");"
+                     "}";
+
+  test(driver, prog, 0);
+
+  std::vector<int> sizes = { 1, 2, 4, 8, 8, 8 };
+
+  for (size_t i = 0; i < sizes.size(); i++)
+  {
+    auto v = static_cast<ast::AssignVarStatement *>(
+        driver.root_->probes->at(0)->stmts->at(i));
+    EXPECT_EQ(SizedType(Type::integer, 8, false), v->var->type);
+    EXPECT_EQ(true, v->var->type.is_pointer);
+    EXPECT_EQ((unsigned long int)sizes.at(i), v->var->type.pointee_size);
+  }
 }
 
 TEST(semantic_analyser, call_reg)
@@ -535,18 +594,14 @@ TEST(semantic_analyser, call_func)
 {
   test("kprobe:f { @[func] = count(); }", 0);
   test("kprobe:f { printf(\"%s\", func);  }", 0);
-  test("kprobe:f { func(\"blah\"); }", 1);
-  test("kprobe:f { func(); }", 1);
-  test("kprobe:f { func(123); }", 1);
+  test("uprobe:/bin/sh:f { @[func] = count(); }", 0);
+  test("uprobe:/bin/sh:f { printf(\"%s\", func);  }", 0);
 }
 
 TEST(semantic_analyser, call_probe)
 {
   test("kprobe:f { @[probe] = count(); }", 0);
   test("kprobe:f { printf(\"%s\", probe);  }", 0);
-  test("kprobe:f { probe(\"blah\"); }", 1);
-  test("kprobe:f { probe(); }", 1);
-  test("kprobe:f { probe(123); }", 1);
 }
 
 TEST(semantic_analyser, call_cat)
@@ -560,6 +615,44 @@ TEST(semantic_analyser, call_cat)
   test("kprobe:f { @[cat(\"/proc/loadavg\")] = 1; }", 1);
 }
 
+TEST(semantic_analyser, call_stack)
+{
+  test("kprobe:f { kstack() }", 0);
+  test("kprobe:f { ustack() }", 0);
+  test("kprobe:f { kstack(bpftrace) }", 0);
+  test("kprobe:f { ustack(bpftrace) }", 0);
+  test("kprobe:f { kstack(perf) }", 0);
+  test("kprobe:f { ustack(perf) }", 0);
+  test("kprobe:f { kstack(3) }", 0);
+  test("kprobe:f { ustack(3) }", 0);
+  test("kprobe:f { kstack(perf, 3) }", 0);
+  test("kprobe:f { ustack(perf, 3) }", 0);
+
+  // Wrong arguments
+  test("kprobe:f { kstack(3, perf) }", 10);
+  test("kprobe:f { ustack(3, perf) }", 10);
+  test("kprobe:f { kstack(perf, 3, 4) }", 1);
+  test("kprobe:f { ustack(perf, 3, 4) }", 1);
+  test("kprobe:f { kstack(bob) }", 1);
+  test("kprobe:f { ustack(bob) }", 1);
+  test("kprobe:f { kstack(\"str\") }", 10);
+  test("kprobe:f { ustack(\"str\") }", 10);
+  test("kprobe:f { kstack(perf, \"str\") }", 10);
+  test("kprobe:f { ustack(perf, \"str\") }", 10);
+  test("kprobe:f { kstack(\"str\", 3) }", 10);
+  test("kprobe:f { ustack(\"str\", 3) }", 10);
+
+  // Non-literals
+  test("kprobe:f { @x = perf; kstack(@x) }", 10);
+  test("kprobe:f { @x = perf; ustack(@x) }", 10);
+  test("kprobe:f { @x = perf; kstack(@x, 3) }", 10);
+  test("kprobe:f { @x = perf; ustack(@x, 3) }", 10);
+  test("kprobe:f { @x = 3; kstack(@x) }", 10);
+  test("kprobe:f { @x = 3; ustack(@x) }", 10);
+  test("kprobe:f { @x = 3; kstack(perf, @x) }", 10);
+  test("kprobe:f { @x = 3; ustack(perf, @x) }", 10);
+}
+
 TEST(semantic_analyser, map_reassignment)
 {
   test("kprobe:f { @x = 1; @x = 2; }", 0);
@@ -597,9 +690,6 @@ TEST(semantic_analyser, variables_are_local)
 TEST(semantic_analyser, array_access) {
   test("kprobe:f { $s = arg0; @x = $s->y[0];}", 10);
   test("kprobe:f { $s = 0; @x = $s->y[0];}", 10);
-  test("struct MyStruct { int y[4]; } kprobe:f { $s = (struct MyStruct *) "
-       "arg0; @x = $s->y[0];}",
-       0);
   test("struct MyStruct { int y[4]; } kprobe:f { $s = (struct MyStruct *) "
        "arg0; @x = $s->y[5];}",
        10);
@@ -609,6 +699,18 @@ TEST(semantic_analyser, array_access) {
   test("struct MyStruct { int y[4]; } kprobe:f { $s = (struct MyStruct *) "
        "arg0; @x = $s->y[\"0\"];}",
        10);
+  test("struct MyStruct { int y[4]; } kprobe:f { $s = (struct MyStruct *) "
+       "arg0; $idx = 0; @x = $s->y[$idx];}",
+       10);
+  BPFtrace bpftrace;
+  Driver driver(bpftrace);
+  test(driver,
+       "struct MyStruct { int y[4]; } kprobe:f { $s = (struct MyStruct *) "
+       "arg0; @x = $s->y[0];}",
+       0);
+  auto assignment = static_cast<ast::AssignMapStatement *>(
+      driver.root_->probes->at(0)->stmts->at(1));
+  EXPECT_EQ(SizedType(Type::integer, 8, true), assignment->map->type);
 }
 
 TEST(semantic_analyser, variable_type)
@@ -633,7 +735,7 @@ TEST(semantic_analyser, map_integer_sizes)
   BPFtrace bpftrace;
   Driver driver(bpftrace);
   std::string structs = "struct type1 { int x; }";
-  test(driver, structs + "kprobe:f { $x = ((type1)0).x; @x = $x; }", 0);
+  test(driver, structs + "kprobe:f { $x = ((struct type1)0).x; @x = $x; }", 0);
 
   auto var_assignment = static_cast<ast::AssignVarStatement*>(driver.root_->probes->at(0)->stmts->at(0));
   auto map_assignment = static_cast<ast::AssignMapStatement*>(driver.root_->probes->at(0)->stmts->at(1));
@@ -644,16 +746,16 @@ TEST(semantic_analyser, map_integer_sizes)
 TEST(semantic_analyser, unop_dereference)
 {
   test("kprobe:f { *0; }", 0);
-  test("struct X { int n; } kprobe:f { $x = (X*)0; *$x; }", 0);
-  test("struct X { int n; } kprobe:f { $x = (X)0; *$x; }", 1);
+  test("struct X { int n; } kprobe:f { $x = (struct X*)0; *$x; }", 0);
+  test("struct X { int n; } kprobe:f { $x = (struct X)0; *$x; }", 1);
   test("kprobe:f { *\"0\"; }", 10);
 }
 
 TEST(semantic_analyser, unop_not)
 {
   test("kprobe:f { ~0; }", 0);
-  test("struct X { int n; } kprobe:f { $x = (X*)0; ~$x; }", 10);
-  test("struct X { int n; } kprobe:f { $x = (X)0; ~$x; }", 10);
+  test("struct X { int n; } kprobe:f { $x = (struct X*)0; ~$x; }", 10);
+  test("struct X { int n; } kprobe:f { $x = (struct X)0; ~$x; }", 10);
   test("kprobe:f { ~\"0\"; }", 10);
 }
 
@@ -670,9 +772,6 @@ TEST(semantic_analyser, unop_increment_decrement)
   test("kprobe:f { --@x; }", 0);
 
   test("kprobe:f { $x++; }", 1);
-  test("kprobe:f { 0++; }", 1);
-  test("kprobe:f { \"a\"++; }", 1);
-
   test("kprobe:f { @x = \"a\"; @x++; }", 1);
   test("kprobe:f { $x = \"a\"; $x++; }", 10);
 }
@@ -803,6 +902,7 @@ TEST(semantic_analyser, join_delimiter)
   test("kprobe:f { $fmt = \"mystring\"; join($fmt, \",\") }", 10);
   test("kprobe:f { @x = join(arg0, \",\") }", 1);
   test("kprobe:f { $x = join(arg0, \",\") }", 1);
+  test("kprobe:f { join(arg0, 3) }", 10);
 }
 
 TEST(semantic_analyser, kprobe)
@@ -819,6 +919,11 @@ TEST(semantic_analyser, kprobe)
 TEST(semantic_analyser, uprobe)
 {
   test("uprobe:/bin/sh:f { 1 }", 0);
+  test("u:/bin/sh:f { 1 }", 0);
+  test("uprobe:/bin/sh:0x10 { 1 }", 0);
+  test("u:/bin/sh:0x10 { 1 }", 0);
+  test("uprobe:/bin/sh:f+0x10 { 1 }", 0);
+  test("u:/bin/sh:f+0x10 { 1 }", 0);
   test("uprobe:sh:f { 1 }", 0);
   test("uprobe:/notexistfile:f { 1 }", 1);
   test("uprobe:notexistfile:f { 1 }", 1);
@@ -826,7 +931,13 @@ TEST(semantic_analyser, uprobe)
   test("uprobe { 1 }", 1);
 
   test("uretprobe:/bin/sh:f { 1 }", 0);
+  test("ur:/bin/sh:f { 1 }", 0);
   test("uretprobe:sh:f { 1 }", 0);
+  test("ur:sh:f { 1 }", 0);
+  test("uretprobe:/bin/sh:0x10 { 1 }", 0);
+  test("ur:/bin/sh:0x10 { 1 }", 0);
+  test("uretprobe:/bin/sh:f+0x10 { 1 }", 1);
+  test("ur:/bin/sh:f+0x10 { 1 }", 1);
   test("uretprobe:/notexistfile:f { 1 }", 1);
   test("uretprobe:notexistfile:f { 1 }", 1);
   test("uretprobe:f { 1 }", 1);
@@ -907,48 +1018,49 @@ TEST(semantic_analyser, profile)
 TEST(semantic_analyser, variable_cast_types)
 {
   std::string structs = "struct type1 { int field; } struct type2 { int field; }";
-  test(structs + "kprobe:f { $x = (type1)cpu; $x = (type1)cpu; }", 0);
-  test(structs + "kprobe:f { $x = (type1)cpu; $x = (type2)cpu; }", 1);
+  test(structs + "kprobe:f { $x = (struct type1)cpu; $x = (struct type1)cpu; }", 0);
+  test(structs + "kprobe:f { $x = (struct type1)cpu; $x = (struct type2)cpu; }", 1);
 }
 
 TEST(semantic_analyser, map_cast_types)
 {
   std::string structs = "struct type1 { int field; } struct type2 { int field; }";
-  test(structs + "kprobe:f { @x = (type1)cpu; @x = (type1)cpu; }", 0);
-  test(structs + "kprobe:f { @x = (type1)cpu; @x = (type2)cpu; }", 1);
+  test(structs + "kprobe:f { @x = (struct type1)cpu; @x = (struct type1)cpu; }", 0);
+  test(structs + "kprobe:f { @x = (struct type1)cpu; @x = (struct type2)cpu; }", 1);
 }
 
 TEST(semantic_analyser, variable_casts_are_local)
 {
   std::string structs = "struct type1 { int field; } struct type2 { int field; }";
-  test(structs + "kprobe:f { $x = (type1)cpu } kprobe:g { $x = (type2)cpu; }", 0);
+  test(structs + "kprobe:f { $x = (struct type1)cpu } kprobe:g { $x = (struct type2)cpu; }", 0);
 }
 
 TEST(semantic_analyser, map_casts_are_global)
 {
   std::string structs = "struct type1 { int field; } struct type2 { int field; }";
-  test(structs + "kprobe:f { @x = (type1)cpu } kprobe:g { @x = (type2)cpu; }", 1);
+  test(structs + "kprobe:f { @x = (struct type1)cpu } kprobe:g { @x = (struct type2)cpu; }", 1);
 }
 
 TEST(semantic_analyser, cast_unknown_type)
 {
-  test("kprobe:f { (faketype)cpu }", 1);
+  test("kprobe:f { (struct faketype)cpu }", 1);
 }
 
 TEST(semantic_analyser, field_access)
 {
   std::string structs = "struct type1 { int field; }";
-  test(structs + "kprobe:f { ((type1)cpu).field }", 0);
-  test(structs + "kprobe:f { $x = (type1)cpu; $x.field }", 0);
-  test(structs + "kprobe:f { @x = (type1)cpu; @x.field }", 0);
+  test(structs + "kprobe:f { ((struct type1)cpu).field }", 0);
+  test(structs + "kprobe:f { $x = (struct type1)cpu; $x.field }", 0);
+  test(structs + "kprobe:f { @x = (struct type1)cpu; @x.field }", 0);
+  test("struct task_struct {int x;} kprobe:f { curtask->x }", 0);
 }
 
 TEST(semantic_analyser, field_access_wrong_field)
 {
   std::string structs = "struct type1 { int field; }";
-  test(structs + "kprobe:f { ((type1)cpu).blah }", 1);
-  test(structs + "kprobe:f { $x = (type1)cpu; $x.blah }", 1);
-  test(structs + "kprobe:f { @x = (type1)cpu; @x.blah }", 1);
+  test(structs + "kprobe:f { ((struct type1)cpu).blah }", 1);
+  test(structs + "kprobe:f { $x = (struct type1)cpu; $x.blah }", 1);
+  test(structs + "kprobe:f { @x = (struct type1)cpu; @x.blah }", 1);
 }
 
 TEST(semantic_analyser, field_access_wrong_expr)
@@ -962,22 +1074,22 @@ TEST(semantic_analyser, field_access_types)
   std::string structs = "struct type1 { int field; char mystr[8]; }"
                         "struct type2 { int field; }";
 
-  test(structs + "kprobe:f { ((type1)0).field == 123 }", 0);
-  test(structs + "kprobe:f { ((type1)0).field == \"abc\" }", 10);
+  test(structs + "kprobe:f { ((struct type1)0).field == 123 }", 0);
+  test(structs + "kprobe:f { ((struct type1)0).field == \"abc\" }", 10);
 
-  test(structs + "kprobe:f { ((type1)0).mystr == \"abc\" }", 0);
-  test(structs + "kprobe:f { ((type1)0).mystr == 123 }", 10);
+  test(structs + "kprobe:f { ((struct type1)0).mystr == \"abc\" }", 0);
+  test(structs + "kprobe:f { ((struct type1)0).mystr == 123 }", 10);
 
-  test(structs + "kprobe:f { ((type1)0).field == ((type2)0).field }", 0);
-  test(structs + "kprobe:f { ((type1)0).mystr == ((type2)0).field }", 10);
+  test(structs + "kprobe:f { ((struct type1)0).field == ((struct type2)0).field }", 0);
+  test(structs + "kprobe:f { ((struct type1)0).mystr == ((struct type2)0).field }", 10);
 }
 
 TEST(semantic_analyser, field_access_pointer)
 {
   std::string structs = "struct type1 { int field; }";
-  test(structs + "kprobe:f { ((type1*)0)->field }", 0);
-  test(structs + "kprobe:f { ((type1*)0).field }", 1);
-  test(structs + "kprobe:f { *((type1*)0) }", 0);
+  test(structs + "kprobe:f { ((struct type1*)0)->field }", 0);
+  test(structs + "kprobe:f { ((struct type1*)0).field }", 1);
+  test(structs + "kprobe:f { *((struct type1*)0) }", 0);
 }
 
 TEST(semantic_analyser, field_access_sub_struct)
@@ -985,12 +1097,12 @@ TEST(semantic_analyser, field_access_sub_struct)
   std::string structs = "struct type2 { int field; } "
                         "struct type1 { struct type2 *type2ptr; struct type2 type2; }";
 
-  test(structs + "kprobe:f { ((type1)0).type2ptr->field }", 0);
-  test(structs + "kprobe:f { ((type1)0).type2.field }", 0);
-  test(structs + "kprobe:f { $x = (type2)0; $x = ((type1)0).type2 }", 0);
-  test(structs + "kprobe:f { $x = (type2*)0; $x = ((type1)0).type2ptr }", 0);
-  test(structs + "kprobe:f { $x = (type1)0; $x = ((type1)0).type2 }", 1);
-  test(structs + "kprobe:f { $x = (type1*)0; $x = ((type1)0).type2ptr }", 1);
+  test(structs + "kprobe:f { ((struct type1)0).type2ptr->field }", 0);
+  test(structs + "kprobe:f { ((struct type1)0).type2.field }", 0);
+  test(structs + "kprobe:f { $x = (struct type2)0; $x = ((struct type1)0).type2 }", 0);
+  test(structs + "kprobe:f { $x = (struct type2*)0; $x = ((struct type1)0).type2ptr }", 0);
+  test(structs + "kprobe:f { $x = (struct type1)0; $x = ((struct type1)0).type2 }", 1);
+  test(structs + "kprobe:f { $x = (struct type1*)0; $x = ((struct type1)0).type2ptr }", 1);
 }
 
 TEST(semantic_analyser, field_access_is_internal)
@@ -999,11 +1111,11 @@ TEST(semantic_analyser, field_access_is_internal)
   Driver driver(bpftrace);
   std::string structs = "struct type1 { int x; }";
 
-  test(driver, structs + "kprobe:f { $x = ((type1)0).x }", 0);
+  test(driver, structs + "kprobe:f { $x = ((struct type1)0).x }", 0);
   auto var_assignment1 = static_cast<ast::AssignVarStatement*>(driver.root_->probes->at(0)->stmts->at(0));
   EXPECT_EQ(false, var_assignment1->var->type.is_internal);
 
-  test(driver, structs + "kprobe:f { @type1 = (type1)0; $x = @type1.x }", 0);
+  test(driver, structs + "kprobe:f { @type1 = (struct type1)0; $x = @type1.x }", 0);
   auto map_assignment = static_cast<ast::AssignMapStatement*>(driver.root_->probes->at(0)->stmts->at(0));
   auto var_assignment2 = static_cast<ast::AssignVarStatement*>(driver.root_->probes->at(0)->stmts->at(1));
   EXPECT_EQ(true, map_assignment->map->type.is_internal);
@@ -1102,7 +1214,7 @@ TEST(semantic_analyser, signed_int_arithmetic_warnings)
 TEST(semantic_analyser, signed_int_division_warnings)
 {
   bool invert = true;
-  std::string msg = "signed division";
+  std::string msg = "signed operands";
   test_for_warning("kprobe:f { @ = -1 / 1 }", msg);
   test_for_warning("kprobe:f { @ = 1 / -1 }", msg);
 
@@ -1111,6 +1223,18 @@ TEST(semantic_analyser, signed_int_division_warnings)
   test_for_warning("kprobe:f { @ = -(1 / 1) }", msg, invert);
 }
 
+TEST(semantic_analyser, signed_int_modulo_warnings)
+{
+  bool invert = true;
+  std::string msg = "signed operands";
+  test_for_warning("kprobe:f { @ = -1 % 1 }", msg);
+  test_for_warning("kprobe:f { @ = 1 % -1 }", msg);
+
+  // These should not trigger a warning
+  test_for_warning("kprobe:f { @ = 1 % 1 }", msg, invert);
+  test_for_warning("kprobe:f { @ = -(1 % 1) }", msg, invert);
+}
+
 TEST(semantic_analyser, map_as_lookup_table)
 {
   // Initializing a map should not lead to usage issues
@@ -1125,7 +1249,7 @@ TEST(semantic_analyser, cast_sign)
   std::string prog =
     "struct t { int s; unsigned int us; long l; unsigned long ul }; "
     "kprobe:f { "
-    "  $t = ((t *)0xFF);"
+    "  $t = ((struct t *)0xFF);"
     "  $s = $t->s; $us = $t->us; $l = $t->l; $lu = $t->ul; }";
   test(driver, prog, 0);
 
@@ -1145,7 +1269,7 @@ TEST(semantic_analyser, binop_sign)
   std::string prog_pre =
     "struct t { long l; unsigned long ul }; "
     "kprobe:f { "
-    "  $t = ((t *)0xFF); ";
+    "  $t = ((struct t *)0xFF); ";
 
   std::string operators[] = { "==", "!=", "<", "<=", ">", ">=", "+", "-", "/", "*"};
   for(std::string op : operators)
@@ -1222,6 +1346,96 @@ TEST(semantic_analyser, intptr_cast_usage)
   test("kprobe:f { @=avg(*(int32*)123) }", 0);
 }
 
+TEST(semantic_analyser, signal)
+{
+  // int literals
+  test("k:f { signal(1); }", 0, false);
+  test("kr:f { signal(1); }", 0, false);
+  test("u:/bin/sh:f { signal(11); }", 0, false);
+  test("ur:/bin/sh:f { signal(11); }", 0, false);
+  test("p:hz:1 { signal(1); }", 0, false);
+
+  // vars
+  test("k:f { @=1; signal(@); }", 0, false);
+  test("k:f { @=1; signal((int32)arg0); }", 0, false);
+
+  // String
+  test("k:f { signal(\"KILL\"); }", 0, false);
+  test("k:f { signal(\"SIGKILL\"); }", 0, false);
+
+  // Not allowed for:
+  test("hardware:pcm:1000 { signal(1); }", 1, false);
+  test("software:pcm:1000 { signal(1); }", 1, false);
+  test("BEGIN { signal(1); }", 1, false);
+  test("END { signal(1); }", 1, false);
+  test("i:s:1 { signal(1); }", 1, false);
+
+  // invalid signals
+  test("k:f { signal(0); }", 1, false);
+  test("k:f { signal(-100); }", 1, false);
+  test("k:f { signal(100); }", 1, false);
+  test("k:f { signal(\"SIGABC\"); }", 1, false);
+  test("k:f { signal(\"ABC\"); }", 1, false);
+
+  // Missing kernel support
+  MockBPFfeature feature(false);
+  test(feature, "k:f { signal(1) }", 1, false);
+  test(feature, "k:f { signal(\"KILL\"); }", 1, false);
+}
+
+TEST(semantic_analyser, strncmp)
+{
+  // Test strncmp builtin
+  test("i:s:1 { $a = \"bar\"; strncmp(\"foo\", $a, 1) }", 0);
+  test("i:s:1 { strncmp(\"foo\", \"bar\", 1) }", 0);
+  test("i:s:1 { strncmp(1) }", 1);
+  test("i:s:1 { strncmp(1,1,1) }", 10);
+  test("i:s:1 { strncmp(\"a\",1,1) }", 10);
+  test("i:s:1 { strncmp(\"a\",\"a\",-1) }", 1);
+  test("i:s:1 { strncmp(\"a\",\"a\",\"foo\") }", 1);
+}
+
+TEST(semantic_analyser, override)
+{
+  // literals
+  test("k:f { override(-1); }", 0, false);
+
+  // variables
+  test("k:f { override(arg0); }", 0, false);
+
+  // Probe types
+  test("kr:f { override(-1); }", 1, false);
+  test("u:f { override(-1); }", 1, false);
+  test("t:syscalls:sys_enter_openat { override(-1); }", 1, false);
+  test("i:s:1 { override(-1); }", 1, false);
+  test("p:hz:1 { override(-1); }", 1, false);
+}
+
+TEST(semantic_analyser, struct_member_keywords)
+{
+  std::string keywords[] = {
+    "arg0", "args", "curtask", "func", "gid" "rand", "uid",
+    "avg", "cat", "exit", "kaddr", "min", "printf", "usym",
+    "kstack", "ustack", "bpftrace", "perf", "uprobe", "kprobe",
+  };
+  for(auto kw : keywords)
+  {
+    test("struct S{ int " + kw + ";}; k:f { ((struct S*)arg0)->" + kw + "}", 0);
+    test("struct S{ int " + kw + ";}; k:f { ((struct S)arg0)." + kw + "}", 0);
+  }
+}
+
+TEST(semantic_analyser, builtin_args)
+{
+  auto bpftrace = get_mock_bpftrace();
+  test(*bpftrace, "t:sched:sched_one { args->common_field }", 0);
+  test(*bpftrace, "t:sched:sched_two { args->common_field }", 0);
+  test(*bpftrace, "t:sched:sched_one,"
+                  "t:sched:sched_two { args->common_field }", 0);
+  test(*bpftrace, "t:sched:sched_* { args->common_field }", 0);
+  test(*bpftrace, "t:sched:sched_one { args->not_a_field }", 1);
+}
+
 } // namespace semantic_analyser
 } // namespace test
 } // namespace bpftrace
